
# HalTree Purpose:
#
# HalTree is a way to organize the mess of data you get from hal.  In general,
# if you want to get all the information about every device in the system, you
# end up with a list of dicts, where each dict contains the property name/values
# for a device.  This list isn't very useful as the hal data is actually 
# organized into a tree.  For example, you have the computer as the head, then
# there may be a scsi card plugged in.  That in turn will have scsi channels
# and luns, which scsi devices may be connected to.  So this module will help
# you reorganize your hal data back to the way they were intended.
#
# HalTree Usage:
# 
# The tree gets built one device at a time.  Once you've created a HalTree
# object, devices get added to the tree with HalTree.add(hw_dev_dict).  The
# devices can be added in any particular order, and the tree gets properly
# structured as the devices get added.  But the tree structure isn't likely
# to be ready until all the devices have been added.  Those devices without a
# parent get stuck in the no_parent_yet list.
#
# When a device gets added, it is no longer a plain dict.  It is stored in a
# HalDevice.  The original dict can be found in HalDevice.properties.

import types


class HalDevice:
    "An object containing its udi, properties and children"
    def __init__ (self, properties):
        self.udi = properties['info.udi']

        self.properties = properties
        self.children = []
        self.classification = None
        
        if properties.has_key('info.parent'):
            self.parent_udi = properties['info.parent']
        else:
            self.parent_udi = None

        self.parent = None

    def print_properties(self):
        print self.udi, ":"
        for property, value in self.properties.items():
            print "    ", property," ==> ",  value

    
        

class HalTree:
    def __init__ (self):
        self.head = None
        self.no_parent_yet = []


    def add(self, hal_device):
        if hal_device.parent_udi:
            parent = self.__find_node(hal_device.parent_udi)
            if parent:
                parent.children.append(hal_device)
                hal_device.parent = parent
            else:  #parent isn't in the main tree yet, stick it in waiting
                self.no_parent_yet.append(hal_device)
        else: #if it doesn't have a parent, it must be the head 'computer'
            self.head = hal_device
            
        #check to see if there are any children waiting for this dev
        self.__get_lost_children(hal_device)
            

    def __get_lost_children(self, hal_device):
        found_list = []
        indexes = []
        no_parent_yet_copy = self.no_parent_yet[:]
        for dev in no_parent_yet_copy:
            if dev.parent_udi == hal_device.udi:
                dev.parent = hal_device
                hal_device.children.append(dev)
                self.no_parent_yet.remove(dev)

    def __find_node(self, udi):
        """ 
        This takes a node in the HalDevice tree and returns the HalDevice with
        the given udi.
        """
        if self.head:
            node = HalTree.__find_node_worker(self.head, udi)
            if node:
                return node

        for node in self.no_parent_yet:
            found_node = HalTree.__find_node_worker(node, udi)
            if found_node:
                return found_node
        return None

    @staticmethod
    def __find_node_worker(node, udi):
        if node.udi == udi:
            return node
        for device in node.children:
            res = HalTree.__find_node_worker(device, udi)
            if res:
                return res
        return None
    
    def print_tree(self):
        self.__print_dev_tree(self.head, "")
        
    def __print_dev_tree(self, node, indent):
        print indent, node.udi
        print indent, "CLASS:", node.classification
        for name, property in node.properties.items():
            if (type(property) == types.StringType):
                if property.isdigit():
                    print indent + "    ", "%-20s ==> %s" % (name, hex(int(property)))
                else:
                    print indent + "    ", "%-20s ==> %s" % (name, property)
            elif (type(property) == types.IntType): 
                print indent + "    ", "%-20s ==> %s" % (name, hex(int(property)))
            else:
                print indent + "    ", "%-20s ==> %s" % (name, property)
        print
        for child in node.children:
            self.__print_dev_tree(child, indent + "    ")



# rhn_register.py - GUI front end code for firstboot screen resolution
#
# Copyright 2003 Red Hat, Inc.
# Copyright 2003 Brent Fox <bfox@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import gtk
import gobject
import sys
import os
import functions

import gnome, gnome.ui
from gtk import glade

from rhn_register_firstboot_gui_window import RhnRegisterFirstbootGuiWindow
sys.path.insert(0, "/usr/share/rhn/up2date_client/")
sys.path.insert(1,"/usr/share/rhn")

import rhnreg
import rhnregGui

import gettext
t = gettext.translation('rhn-client-tools', fallback=True)
_ = t.ugettext
gtk.glade.bindtextdomain("rhn-client-tools", "/usr/share/locale")



class RhnLoginWindow(RhnRegisterFirstbootGuiWindow, rhnregGui.LoginPage):
    #You must specify a runPriority for the order in which you wish your module to run
    runPriority = 108
    moduleName = _("Red Hat Login")
    windowTitle = moduleName
    shortMessage = _("Register with Red Hat Satellite")
    needsparent = 1
    needsnetwork = 1
    noSidebar = True

    def __init__(self):
        RhnRegisterFirstbootGuiWindow.__init__(self)
        rhnregGui.LoginPage.__init__(self)
        if rhnreg.registered():
            self.skipme = True

    def updatePage(self):
#        self.getCaps()
        self.loginPagePrepare()
        self.goingNextFromNewAccountDialog = False
    
    def grabFocus(self):
        # We must set focus where we want it here. Setting it in updatePage
        # doesn't work.
        self.loginXml.get_widget("loginUserEntry").grab_focus()

    def _getVbox(self):
        return self.loginPageVbox()

    def apply(self, *args):
        """Returns None to stay on the same page. Anything else will cause 
        firstboot to advance but True is generally used. This is different from 
        the gnome druid in rhn_register.
        
        """
        if self.doDebug:
            print "applying rhn_login_gui"
        
        # This is a hack. This function will get called if they click next on
        # the login page (the else) or when they create an account (the if). In 
        # that case we don't want to do the normal logging in stuff.
        if self.goingNextFromNewAccountDialog:
            assert rhnregGui.newAccount is True
        else:
            if self.loginPageVerify():
                return None
            
            assert rhnregGui.newAccount is False
            
            if self.loginPageApply():
                return None

        # We should try to activate hardware, even if no EUS in firstboot
        rhnregGui.try_to_activate_hardware()

        self.parent.setPage('rhn_create_profile_gui')
        return True
    
    def goToPageAfterLogin(self):
        # This is a hack. More info above.
        self.goingNextFromNewAccountDialog = True
        self.parent.nextClicked()
    
    def onLoginPageNext(self, dummy=None, dummy2=None):
        # This is a hackish way to support enter advancing
        self.parent.nextClicked()

childWindow = RhnLoginWindow



#!/usr/bin/python

import sys

import settestpath

import unittest

#Import the modules you need to test...
from up2date_client import clientCaps


class TestClientCaps(unittest.TestCase):
    def testEmptyInit(self):
        "Verify that the class can be created with no arguments"
        cc = clientCaps.ClientCapabilities()

    def testPopulate(self):
        "Verify the object gets created with an approriate populated data"
        cc = clientCaps.ClientCapabilities()
        len = cc.keys()
        self.assertTrue(len >= 1)

    def testHeaderFormat(self):
        "Verify that headerFormat runs without errors"
        cc = clientCaps.ClientCapabilities()
        res = cc.headerFormat()
        self.assertEquals(type([]),  type(res))


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestClientCaps))
    return suite
 
if __name__ == "__main__":
    unittest.main(argv=sys.argv)



#!/usr/bin/python

import sys

import unittest

#Import the modules you need to test...
import httplib

def write(blip):
    sys.stdout.write("\n|%s|\n" % blip)


class TestSomething(unittest.TestCase):
    def setUp(self):
	self.__setupData()

    def __setupData(self):
        pass
        
    def testHttpConnection(self):
        con = httplib.HTTPConnection("www.adrianlikins.com")
        con.request("GET", "/")
        r1 = con.getresponse()
        data = r1.read()
        con.close()

    def testHttpsConnection(self):
        con = httplib.HTTPSConnection("rhn.redhat.com")
        con.request("GET", "/")
        r1 = con.getresponse()
        data = r1.read()
        con.close()

    def testHTTPSConnectionTimeout(self):
        import socket
        socket.setdefaulttimeout(3)
        con = httplib.HTTPSConnection("rhn.redhat.com")
        con.request("GET", "/")
        r1 = con.getresponse()
        data = r1.read()
        con.close() 


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestSomething))
    return suite
 
if __name__ == "__main__":
    unittest.main(argv=sys.argv)



#!/usr/bin/python

import settestpath

from up2date_client import up2dateErrors
##from up2date_client import config

import unittest

#Import the modules you need to test...
from up2date_client import rhnregGui
from up2date_client.rhnregGui import callAndFilterExceptions

class MyError(Exception):
    pass

class TestcallAndFilterExceptions(unittest.TestCase):
    def storeMessage(self, message, dummy):
        self.message = message
    
    def returnsNumber(self):
        return 6
    
    def raiseError(self):
        raise MyError("I am a banana!")
    
    def testCallableDoesntRaiseException(self):
        value = callAndFilterExceptions(self.returnsNumber, [], "Error", 
                                            self.storeMessage)
        self.assertEqual(value, 6)
    
    def testCallableRaisesValidException(self):
        self.assertRaises(MyError,
                        callAndFilterExceptions, self.raiseError, 
                        [MyError], "Error", 
                        self.storeMessage)

    def testCallableRaisesInvalidException(self):
        value = callAndFilterExceptions(self.raiseError, [], "Error3", 
                                            self.storeMessage)
        self.assertEqual(value, None)
        self.assertEqual(self.message, "Error3")
    
#TODO: things we don't test yet but need to
# authToken timeout (need some server magic)
# verifying that the authTokens we get actually work

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestcallAndFilterExceptions))
    return suite

if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# Copyright 2010 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Authors:
#     Jan Pazdziora jpazdziora at redhat dot com

from firstboot.moduleset import *

import gettext
_ = lambda x: gettext.ldgettext("rhn-client-tools", x)

class moduleClass(ModuleSet):
    def __init__(self):
        ModuleSet.__init__(self)
        self.priority = 3
        self.sidebarTitle = _("Set Up Software Updates")
        self.title = _("Set Up Software Updates")
        self.path = "/usr/share/rhn/up2date_client/firstboot"




#!/usr/bin/python

import settestpath

from up2date_client import up2dateUtils

import unittest

TestCase = unittest.TestCase
test_up2date = "etc-sysconfig-rhn/up2date"

class ReturnsString(TestCase):
    def testReturnsString(self):
        "Verify that function returns string"
        assert type(self.function()) == type("")

    def testNonZoreLength(self):
        "Verify that function returns a non zero length string"
        assert len(self.function()) > 0

class TestGetVersion(ReturnsString):
    def setUp(self):
        from up2date_client import config
        self.cfg = config.initUp2dateConfig(test_up2date)
        self.function = up2dateUtils.getVersion

    def testVersionOverride(self):
        "Verify that specify a version overide works"
        self.cfg['versionOverride'] = "100"
        res = up2dateUtils.getVersion()
        assert res == "100"

class TestGetOSRelease(ReturnsString):
    def setUp(self):
        self.function = up2dateUtils.getOSRelease

class TestGetRelease(ReturnsString):
    def setUp(self):
        self.function = up2dateUtils.getRelease

class TestGetArch(ReturnsString):
    def setUp(self):
        self.function = up2dateUtils.getArch

    def testIa32eOverride(self):
        "Verify that function does not return ia32e #216225"
        arch = self.function()
        assert arch.find('ia32e') == -1

class TestVersion(ReturnsString):
    def setUp(self):
        self.function = up2dateUtils.version

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestGetVersion))
    suite.addTest(unittest.makeSuite(TestGetOSRelease))
    suite.addTest(unittest.makeSuite(TestGetRelease))
    suite.addTest(unittest.makeSuite(TestGetArch))
    suite.addTest(unittest.makeSuite(TestVersion))
    return suite

if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# Copyright 2006--2010 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Authors:
#     Jan Pazdziora jpazdziora at redhat dot com
#     Daniel Benamy <dbenamy@redhat.com>

import sys
sys.path.append("/usr/share/rhn")
from up2date_client import rhnreg
from up2date_client import rhnregGui

import gtk
from gtk import glade

import gettext
_ = lambda x: gettext.ldgettext("rhn-client-tools", x)

gtk.glade.bindtextdomain("rhn-client-tools")

from firstboot.module import Module
from firstboot.constants import *

from firstboot.loader import _haveNetwork

class moduleClass(Module):
    def __init__(self):
        Module.__init__(self)
        self.priority = 106
        self.sidebarTitle = _("Set Up Software Updates")
        self.title = _("Set Up Software Updates")
        self.skip_registration = False
        self.start_page_vbox = None

    def apply(self, interface, testing=False):
        if testing:
            return RESULT_SUCCESS

        if self.skip_registration:
            interface.moveToPage(pageNum = len(interface.moduleList))
            return RESULT_JUMP

        if not self.start_page.startPageRegisterNow():
            dlg = rhnregGui.ConfirmQuitDialog()
            if dlg.rc == 0:
                return RESULT_FAILURE
            else:
                interface.moveToPage(moduleTitle=_("Finish Updates Setup"))
                return RESULT_JUMP
        return RESULT_SUCCESS

    def createScreen(self):
        self.vbox = gtk.VBox(spacing=5)

    def initializeUI(self):
        if self.start_page_vbox:
            self.start_page_vbox.destroy()

        self.start_page_vbox = self._getVbox()
        self.vbox.pack_start(self.start_page_vbox, True, True)

    def _system_is_registered(self):
        if rhnreg.registered():
            return True
        try:
            _rhsm_path = "/usr/share/rhsm/subscription_manager"
            _rhsm_path_added = False
            if _rhsm_path not in sys.path:
                sys.path.append(_rhsm_path)
                _rhsm_path_added = True
            import certlib
            if _rhsm_path_added:
                sys.path.remove(_rhsm_path)
            return certlib.ConsumerIdentity.existsAndValid()
        except:
            return False

    def _getVbox(self):
        if self._system_is_registered():
            self.start_page = KsRegisteredPage()
            self.skip_registration = True
            return self.start_page.startPageVbox()
        if _haveNetwork():
            self.start_page = rhnregGui.StartPage(firstboot=True)
        else:
            self.start_page = NoNetworkPage()
            self.skip_registration = True
        return self.start_page.startPageVbox()
    
class KsRegisteredPage:

    def __init__(self):
        gladefile = "/usr/share/rhn/up2date_client/rh_register.glade"
        ksRegisteredXml = gtk.glade.XML(gladefile, "ksRegisteredFirstbootVbox",
              domain="rhn-client-tools")
        self.vbox = ksRegisteredXml.get_widget('ksRegisteredFirstbootVbox')

    def startPageVbox(self):
        return self.vbox

    def startPageRegisterNow(self):
        return True


class NoNetworkPage:

    def __init__(self):
        gladefile = "/usr/share/rhn/up2date_client/rh_register.glade"
        noNetworkXml = gtk.glade.XML(gladefile, "noNetworkFirstbootVbox",
            domain="rhn-client-tools")
        self.vbox = noNetworkXml.get_widget('noNetworkFirstbootVbox')
        noNetworkXml.signal_autoconnect({
            "on_whyRegisterButton_clicked" : self.why_register_button_clicked,
        })

    def startPageVbox(self):
        return self.vbox

    def startPageRegisterNow(self):
        # Sure, we'll register now. heh heh heh
        # Just continue on past the rhn stuff.
        return True

    def why_register_button_clicked(self, button):
        rhnregGui.WhyRegisterDialog()




#!/usr/bin/python

import os,sys
import getopt
import string
import shutil
import glob


DBPATH="/tmp/testdb"
DBCACHEPATH="/tmp/testdbcaches/"
UP2DATE_COMMAND="up2date --justdb --dbpath $DBPATH"
DATAPATH="/tmp/datadir"
TOPDIR="../"
DBDIR="%s/testdbs" % "/usr/src/rhn/test/up2date/depsolve/" 
PKGDIR="%s/testpackages" % "/usr/src/rhn/test/up2date/rollbacks/"
RESULTSPATH="%s/results/" % (TOPDIR)
CONFIGPATH="%s/configs" % "/usr/src/rhn/test/up2date/depsolve/"
REPACKAGEDIR="/tmp/testrepackage"
PLATFORMPATH="/etc/rpm/platform"

def createDbDir():
    # remove the old if its there
    try:
        shutil.rmtree(DBPATH)
    except OSError:
        #whatever...
        print "%s doesnt exist, creating it" % DBPATH
        pass
    
    # make the new
    if not os.access(DBPATH, os.W_OK):
        os.makedirs(DBPATH)

def createRepackageDir():
   # remove the old if its there
    try:
        shutil.rmtree(REPACKAGEDIR)
    except OSError:
        #whatever...
        print "%s doesnt exist, creating it" % REPACKAGEDIR
        pass
    
    # make the new
    if not os.access(REPACKAGEDIR, os.W_OK):
        os.makedirs(REPACKAGEDIR)



def createDataDirs():
    createDbDir()
    createRepackageDir()
        
    if not os.access(DATAPATH, os.W_OK):
        os.makedirs(DATAPATH)


def rebuildRepackageDir(repackageName):
    createRepackageDir()
    files = glob.glob("%s/%s/*.rpm" % (PKGDIR, repackageName))
    for file in files: 
    	shutil.copy(file , REPACKAGEDIR)


# fetch a copy of the rebuild db from the cache if we have it
def lookForDbCache(dbname):
    if not os.access("%s/%s" % (DBCACHEPATH, dbname), os.R_OK):
	return 1

    #print "Using db cache"
    files = glob.glob("%s/%s/*" % (DBCACHEPATH, dbname))
    for file in files:
	shutil.copy(file, DBPATH)

    return 0

def populateDbCache(dbname):
    cache = "%s/%s" % (DBCACHEPATH, dbname)
    os.makedirs(cache)
    files = glob.glob("%s/*" % DBPATH)
    for file in files:
	shutil.copy(file, cache)

def rebuildRpmDatabase(dbname):
    createDbDir()
	
    if lookForDbCache(dbname):
	print "Rebuilding rpm database"
    	shutil.copy("%s/%s/Packages" % (DBDIR,dbname) , DBPATH)
    	cmdline = "rpm -v --dbpath %s  --rebuilddb" % DBPATH
    	fd = os.popen(cmdline)
    	fd.read()
    	fd.close()
	
	populateDbCache(dbname)




def buildUp2dateCommand(options):
    ret = "up2date --justdb --dbpath %s %s" % (DBPATH, options)
    return ret

def getRpmQAList():
    cmdline = "rpm --dbpath %s -qa" % DBPATH
    fd = os.popen(cmdline)
    tmp = fd.readlines()
    out = map(lambda a:string.strip(a), tmp)
    
    fd.close()
    return out

def runUp2date(cmd):
    fd = os.popen(cmd)
    ret = fd.read()
    fd.close()
    return ret

def storeResults(results,testname, type):
    if type == "pre":
        fd = open("%s/%s.pre" % (DATAPATH,testname), "w")
        fd.write(string.join(results,"\n"))
        fd.close()
    if type == "after":
        fd = open("%s/%s.post" % (DATAPATH,testname), "w")
        fd.write(string.join(results, "\n"))
        fd.close()

def saveUp2dateOut(up2dateOut,testname):
    fd = open("%s/%s.up2date-out" % (DATAPATH,testname), "w")
    fd.write(up2dateOut)
    fd.close()

FILENAMES = ['/etc/sysconfig/rhn/up2date',
            '/etc/sysconfig/rhn/systemid',
            '/etc/sysconfig/rhn/sources',
            '/etc/sysconfig/rhn/network']


def setupConfig(configname):
    # copy over the approriate up2date and systemid
    # to /etc/sysconfig/rhn

    path = "%s/%s" % (CONFIGPATH, configname)
    for filename in FILENAMES:
        # store a backup of the original if it doesnt exist yet
        if os.access(filename, os.R_OK) and \
               not os.access("%s..orig-test" % filename, os.R_OK):
            shutil.copy(filename, "%s.orig-test" % filename)

	stored = "%s/%s" % (path, os.path.basename(filename))
    	if os.access(stored, os.R_OK):
            shutil.copy(stored, filename)


def restoreConfig():
    for filename in FILENAMES:
    	if os.access("%s.orig-test" % filename, os.R_OK):
            shutil.copy("%s.orig-test" % filename, filename)

def logFailures(name):
    fd = open("%s/FAILURES" % DATAPATH, "w+")
    fd.write("%s\n" % name)
    fd.close()
    
def runTestcase(testcase):
    print "Generating an rpm db in %s based on %s" % (DBPATH, "%s/%s" % (DBDIR,testcase.dbname))
    rebuildRpmDatabase(testcase.dbname)
    cmd = buildUp2dateCommand(testcase.options)
    beforeList = getRpmQAList()
    storeResults(beforeList, testcase.name, "pre")
    setupConfig(testcase.configs)
    print "running up2date as:\n%s" % cmd
    up2dateOut = runUp2date(cmd)
    saveUp2dateOut(up2dateOut,testcase.name)
    afterList = getRpmQAList()
    storeResults(afterList, testcase.name, "after")
    print "diff between before/after"
    compareBeforeAfter(beforeList, afterList)
    print "diff between results and expected results"
    try:
        ret = compareResults(testcase.results, afterList)
    except "NoResultsError":
        print "\n\nNo results listing (%s) was found for this test (%s)\n\n" % (testcase.results, testcase.name)
        ret = 1
    print ret
    if ret:
        print "\n----------- This Case (%s) Failed --------------" % testcase.name
        print "cmd"
        print "dbname: %s results: %s" % (testcase.dbname, testcase.results)
        print "\n\n"
        logFailures(testcase.name)
#    restoreConfig()

def difflists(list1, list2):
    in1_not_in2 = []
    in2_not_in1 = []
    for i in list1:
        if i not in list2:
            in1_not_in2.append(i)

    for i in list2:
        if i not in list1:
            in2_not_in1.append(i)

    return (in1_not_in2, in2_not_in1)

def compareBeforeAfter(before, after):
#    print before
#    print after
    deleted, added = difflists(before, after)
    print "added: %s" % added
    print "deleted: %s" % deleted


def compareResults(resultsName, afterList):
    #open the results file and read it in
    resultsFile = "%s/%s" % (RESULTSPATH, resultsName)
    if not os.access(resultsFile, os.R_OK):
        raise "NoResultsError"
    
    fd = open("%s/%s" % (RESULTSPATH, resultsName), "r")
    expected = fd.readlines()
    expected.sort()
    tmp = map(lambda a:string.strip(a), expected)
    expected = tmp
    afterList.sort()
    deleted, added = difflists(afterList, expected)
    print "epected but not found: %s" % added
    print "not expected, but found: %s" % deleted
    if len(deleted) == 0 and len(added) == 0:
        return 0
    else:
        return 1

            


    
class Testcase:
    def __init__(self, name=None, dbname=None,
                 configs=None, results=None, options=None):
        self.name = name
        self.dbname = dbname
        self.configs = configs
        self.options = options
        self.results = results
    def __repr__(self):
        out = ""
        out = out + "name: %s " % self.name
        out = out + "dbname: %s " % self.dbname
        out = out + "configs: %s " % self.configs
        out = out + "results: %s" % self.results
        out = out + "options: %s " % self.options
        return out
    
testcases = []
testcasenames = []
def parsefile(m):
    while 1:
        s = m.readline()
        if not len(s):
            break    
        s = s[:-1]
        if not s:
            continue
        s = string.strip(s) # remove whitespace
        if s[0] == '#':
            continue
        line = string.split(s, ':')
        testcasename = string.strip(line[0])
        dbname = string.strip(line[1])
        configs = string.strip(line[2])
        results = string.strip(line[3])
        options = string.strip(line[4])

        testcase = Testcase(testcasename, dbname, configs, results, options)
        testcases.append(testcase)
        testcasenames.append(testcase.name)


def main():
    testcasename = None
    printlist = None

    opts, args = getopt.getopt(sys.argv[1:], "f:l",
                               ["filename=", "list"])
    for opt, val in opts:
        if opt == "--filename" or opt == "-f":
            filename = val
        if opt == "--list" or opt == "-l":
            printlist = 1
            print "Available test cases include:"
            print testcasenames
            for i in testcasenames:
                print "%s" % i

    #testcasename = args

    fd = open(filename, "r")
    parsefile(fd)
    #print testcases

    if printlist:
        print "Available test cases include:"
        for i in testcasenames:
            print "%s" % i
        sys.exit()

    createDataDirs()

    # accept a list of test cases, including filename style globs
    if args:
        import fnmatch
        matchedtestcases = []
        for testcasename in args:
            for testcase in testcasenames:
                if fnmatch.fnmatch(testcase, testcasename):
                    matchedtestcases.append(testcase)

        #uniq the list
        tmp = {}
        for i in matchedtestcases:
            tmp[i] = i

        matchedtestcases = tmp.keys()
        matchedtestcases.sort()

        for testcase in matchedtestcases:
            print "going to run testcases: %s" % testcase

        for testcase in matchedtestcases:
            print "running testcase: %s" % testcase
            runTestcase(testcases[testcasenames.index(testcase)])

    if not testcasenames:
        for testcase in testcases:
            runTestcase(testcase)
    #else:
    #    print "running testcase: %s" % testcasename
    #    runTestcase(testcases[testcasenames.index(testcasename)])





#!/usr/bin/python

import sys
import rpm
sys.path.append("../")

import settestpath

from up2date_client import transaction
# lots of useful util methods for building/tearing down
# test enviroments...
import testutils

import unittest

from up2date_client import up2dateAuth

def write(blip):
    sys.stdout.write("\n|%s|\n" % blip)

class TestGetAllPkgInfo(unittest.TestCase):
    def __allPkgs(self, count, multilib=None, arch1=None, arch2=None):
        self.allPkgs = []
	if not arch1:
	    arch1 = "noarch"
        for i in range(count):
            if multilib:
                self.allPkgs.append(["a%04d" % i, '1.0', '1', '', arch1])
                self.allPkgs.append(["a%04d" % i, '1.0', '1', '', arch2])
            else:		
                self.allPkgs.append(["a%04d" % i, '1.0', '1', '', arch1])

    def setUp(self):
        self.__allPkgs(5, multilib=1, arch1='i386', arch2='x86_64')
        self.correctChoice = ['a0001', '1.0', '1', '', 'x86_64']

#    def testGetAllPkgInfoNoArch(self):
#        res = up2date.getAllPkgInfo(self.allPkgs, ['a0001', '1.0', '1', '', ''])
#        write("res:  %s" % res)
#        self.assertEqual(self.correctChoice, res)

class TestTransactionData(unittest.TestCase):
    def setUp(self):
        self.__testData()
        self.td = transaction.TransactionData()

    def __testData(self):
        self.foo = 1
        self.packages1 = [(('kernel', '2.4.0', '1.0', ''), "u"),
                          (('blippy', '1.0', '1.0', '', 'i686'), "i")]

        
    def testPopulateData1(self):
        "Verify that populating data set 1 works correctly"
        self.td.data['packages'] = self.packages1

    def testPrintData1(self):
        "Verify that the string repr of the transaction of data1 is correct"
        self.td.data['packages'] = self.packages1
        res = self.td.display()
        expected =  """\t\t[i] blippy-1.0-1.0:\n\t\t[u] kernel-2.4.0-1.0:\n"""
        self.assertEqual(expected, res)


class TestGenTransaction(unittest.TestCase):
    def setUp(self):
        self.__setupData()

    def __setupData(self):
        self.packages1 = [(('kernel', '2.4.0', '1.0', ''), "u"),
                          (('blippy', '1.0', '1.0', '', 'i686'), "i")]

 
#    def testGenTransactionData1(self):
#        td = transaction.TransactionData()
#        td.data['packages'] = self.packages1

#        res = up2date.genTransaction(td)
#        write(res)
#        print res

class TestGenTransactionSat1(unittest.TestCase):
    setupflag = 0
    def _realSetup(self, testname):
        rpm.addMacro("_dbpath",testutils.DBPATH)
        repackagedir =  "/tmp/testrepackage" 
        rpm.addMacro("_repackage_dir", repackagedir)
        
        if TestGenTransactionSat1.setupflag != 0:
            return
        
        testutils.createDataDirs()
        testutils.rebuildRpmDatabase(testname)
        testutils.rebuildRepackageDir(testname)
        TestGenTransactionSat1.setupflag = 1

    def setUp(self):
        self.__setupData()
        self._realSetup("8.0-workstation-i386-1")
#        testutils.setupConfig("8.0-workstation-i386-1")

    def __setupData(self):
        self.packages1 = [(('wget', '1.8.2', '5', ''), 'u'),
                          (('cvs', '1.11.2', '8', ''), 'u')]
#        self.packages1 = [(('automake', '1.6.3', '1', ''), 'u'),
#                          (('sylpheed', '0.8.2', '1', ''), "u")]
#        self.packages1 = [(('kernel', '2.4.0', '1.0', ''), "u"),
#                          (('blippy', '1.0', '1.0', '', 'i686'), "i")]#

 

    def tearDown(self):
        pass
 #       testutils.restoreConfig()
        

#    def testGenTransactionData1(self):
#        td = transaction.TransactionData()
#        td.data['packages'] = self.packages1

#        res = up2date.genTransaction(td)
#        write(res)
 #       print res

#    def testRunTransactionData1(self):
#        td = transaction.TransactionData()
#        td.data['packages'] = self.packages1

#        res = up2date.genTransaction(td)

        # for rpmCallback...
#        import wrapperUtils
#        rpmCallback = wrapperUtils.RpmCallback()
#        foo = up2date.runTransaction(res, rpmCallback.callback)
#        write(foo)

class TestActionsTransactions(unittest.TestCase):
    setupflag = 0
    def _realSetup(self, testname):
        rpm.addMacro("_dbpath",testutils.DBPATH)
        repackagedir =  "/tmp/testrepackage" 
        rpm.addMacro("_repackage_dir", repackagedir)
        
        if TestGenTransactionSat1.setupflag != 0:
            return
        
        testutils.createDataDirs()
        testutils.rebuildRpmDatabase(testname)
        testutils.rebuildRepackageDir(testname)
        TestGenTransactionSat1.setupflag = 1

    def setUp(self):
        up2dateAuth.updateLoginInfo()
        print up2dateAuth.loginInfo
        self.__setupData()
        self._realSetup("8.0-workstation-i386-1")
#        testutils.setupConfig("8.0-workstation-i386-1")

    def __setupData(self):
        self.packages1 = [(('wget', '1.8.2', '5', ''), 'u'),
                          (('cvs', '1.11.2', '8', ''), 'u')]
 

    def tearDown(self):
        pass

    def testAction1(self):
        from actions import packages

        data = {}
        data['packages'] = self.packages1
        
        res = packages.runTransaction(data)
        write(res)


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestActionsTransactions))
    suite.addTest(unittest.makeSuite(TestGenTransactionSat1))
    suite.addTest(unittest.makeSuite(TestGenTransaction))
    suite.addTest(unittest.makeSuite(TestTransactionData))
    suite.addTest(unittest.makeSuite(TestGetAllPkgInfo))
    return suite

if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# Copyright 2006--2010 Red Hat, Inc.
# Copyright 2003 Brent Fox <bfox@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Authors:
#     Jan Pazdziora jpazdziora at redhat dot com

import sys
sys.path.append("/usr/share/rhn")
from up2date_client import rhnreg
from up2date_client import rhnregGui, rhnserver
from up2date_client import messageWindow

import gtk

import gettext
_ = lambda x: gettext.ldgettext("rhn-client-tools", x)

gtk.glade.bindtextdomain("rhn-client-tools")

from firstboot.module import Module
from firstboot.constants import *

class moduleClass(Module):
    def __init__(self):
        Module.__init__(self)
        self.priority = 108.5
        self.sidebarTitle = _("Red Hat Account")
        self.title = _("Red Hat Account")

    def needsNetwork(self):
        return True

    def apply(self, interface, testing=False):
        if testing:
            return RESULT_SUCCESS

        if self.loginPage.loginPageVerify():
            return RESULT_FAILURE

        ret = self.loginPage.loginPageApply()
        if self.loginPage.go_to_finish:
            interface.moveToPage(moduleTitle=_("Finish Updates Setup"))
            return RESULT_JUMP
        if ret:
            return RESULT_FAILURE

        # We should try to activate hardware, even if no EUS in firstboot
        rhnregGui.try_to_activate_hardware()

        # populate capability - needef for EUSsupported
        s = rhnserver.RhnServer()
        s.capabilities.validate()

        if rhnregGui.ChooseChannelPage().chooseChannelShouldBeShown():
            return RESULT_SUCCESS
        else:
            interface.moveToPage(moduleTitle=_("Create Profile"))
            return RESULT_JUMP

    def createScreen(self):
        self.loginPage = FirstbootLoginPage()
        self.vbox = gtk.VBox(spacing=5)
        self.vbox.pack_start(self.loginPage.loginPageVbox(), True, True)

    def initializeUI(self):
        self.loginPage.loginPagePrepare()

    def shouldAppear(self):
        if rhnreg.registered():
            return False
        return True

    def focus(self):
        self.loginPage.loginXml.get_widget("loginUserEntry").grab_focus()

class FirstbootLoginPage(rhnregGui.LoginPage):
    def __init__(self):
        rhnregGui.LoginPage.__init__(self)

    def loginPageApply(self):
        self.go_to_finish = False
        return rhnregGui.LoginPage.loginPageApply(self)

    def fatalError(self, error, wrap=1):
        msg = _("There was a communication error with the server:") \
            + "\n\n" + error + "\n\n" \
	    + _("Would you like to go back and try again?")
        dlg = messageWindow.YesNoDialog(msg)
        ret = dlg.getrc()
        if not ret:
            self.go_to_finish = True

    def goToPageAfterLogin(self):
        pass

# FIXME: the Enter advancing is not working.
#
#    def goToPageAfterLogin(self):
#        # This is a hack. More info above.
#        self.goingNextFromNewAccountDialog = True
#        self.parent.nextClicked()
#
#    def onLoginPageNext(self, dummy=None, dummy2=None):
#        # This is a hackish way to support enter advancing
#        self.parent.nextClicked()



#
# Copyright (c) 1999--2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

# This thing gets the hardware configuraion out of a system
"""Used to read hardware info from kudzu, /proc, etc"""
from socket import gethostname, getaddrinfo, AF_INET, AF_INET6
import socket
import re
import os
import sys
import config
import rhnserver

try:
    import ethtool
    ethtool_present = True
except ImportError:
    sys.stderr.write("Warning: information about network interfaces could not be retrieved on this platform.\n")
    ethtool_present = False

import gettext
t = gettext.translation('rhn-client-tools', fallback=True)
_ = t.ugettext

import dbus
import dmidecode
import up2dateLog

try: # F13 and EL6
    from hardware_gudev import get_devices, get_computer_info
    using_gudev = 1
except ImportError:
    from hardware_hal import check_hal_dbus_status, get_hal_computer, read_hal
    using_gudev = 0

# Some systems don't have the _locale module installed
try:
    import locale
except ImportError:
    locale = None

sys.path.append("/usr/share/rhsm")
try:
    from subscription_manager.hwprobe import Hardware as SubManHardware
    subscription_manager_available = True
except ImportError:
    subscription_manager_available = False

# this does not change, we can cache it
_dmi_data           = None
_dmi_not_available  = 0

def dmi_warnings():
    if not hasattr(dmidecode, 'get_warnings'):
        return None

    return dmidecode.get_warnings()

dmi_warn = dmi_warnings()
if dmi_warn:
    dmidecode.clear_warnings()
    log = up2dateLog.initLog()
    log.log_debug("Warnings collected during dmidecode import: %s" % dmi_warn)

def _initialize_dmi_data():
    """ Initialize _dmi_data unless it already exist and returns it """
    global _dmi_data, _dmi_not_available
    if _dmi_data is None:
        if _dmi_not_available:
            # do not try to initialize it again and again if not available
            return None
        else :
            dmixml = dmidecode.dmidecodeXML()
            dmixml.SetResultType(dmidecode.DMIXML_DOC)
            # Get all the DMI data and prepare a XPath context
            try:
                data = dmixml.QuerySection('all')
                dmi_warn = dmi_warnings()
                if dmi_warn:
                    dmidecode.clear_warnings()
                    log = up2dateLog.initLog()
                    log.log_debug("dmidecode warnings: " % dmi_warn)
            except:
                # DMI decode FAIL, this can happend e.g in PV guest
                _dmi_not_available = 1
                dmi_warn = dmi_warnings()
                if dmi_warn:
                    dmidecode.clear_warnings()
                return None
            _dmi_data = data.xpathNewContext()
    return _dmi_data

def get_dmi_data(path):
    """ Fetch DMI data from given section using given path.
        If data could not be retrieved, returns empty string.
        General method and should not be used outside of this module.
    """
    dmi_data = _initialize_dmi_data()
    if dmi_data is None:
        return ''
    data = dmi_data.xpathEval(path)
    if data != []:
        return data[0].content
    else:
        # The path do not exist
        return ''

def dmi_vendor():
    """ Return Vendor from dmidecode bios information.
        If this value could not be fetch, returns empty string.
    """
    return get_dmi_data('/dmidecode/BIOSinfo/Vendor')

def dmi_system_uuid():
    """ Return UUID from dmidecode system information.
        If this value could not be fetch, returns empty string.
    """
    # if guest was created manualy it can have empty UUID, in this
    # case dmidecode set attribute unavailable to 1
    uuid = get_dmi_data("/dmidecode/SystemInfo/SystemUUID[not(@unavailable='1')]")
    if not uuid:
        uuid = ''
    return uuid

def read_installinfo():
    if not os.access("/etc/sysconfig/installinfo", os.R_OK):
        return {}
    installinfo = open("/etc/sysconfig/installinfo", "r").readlines()
    installdict = {}
    installdict['class'] = "INSTALLINFO"
    for info in installinfo:
        if not len(info):
            continue
        vals = info.split('=')
        if len(vals) <= 1:
            continue
        strippedstring = vals[0].strip()
        vals[0] = strippedstring
        
        installdict[vals[0]] = ''.join(vals[1:]).strip()
    return installdict
    
def cpu_count():
    """ returns number of CPU in system

    Beware that it can be different from number of active CPU (e.g. on s390x architecture
    """
    try:
        cpu_dir = os.listdir('/sys/devices/system/cpu/')
    except OSError:
        cpu_dir = []

    re_cpu = re.compile(r"^cpu[0-9]+$")
    return len([i for i in cpu_dir if re_cpu.match(i)])

# get the number of sockets available on this machine
def __get_number_sockets():
    try:
        if subscription_manager_available:
            return SubManHardware().getCpuInfo()['cpu.cpu_socket(s)']
    except:
        pass # something went wrong, let's figure it out ourselves

    number_sockets = 0
    # Try lscpu command if available
    if os.access("/usr/bin/lscpu", os.X_OK):
        try:
            lines = os.popen("/usr/bin/lscpu -p").readlines()
            max_socket_index = -1
            for line in lines:
                if line.startswith('#'):
                    continue
                # get the socket index from the output
                socket_index = int(line.split(',')[2])
                if socket_index > max_socket_index:
                    max_socket_index = socket_index
            if max_socket_index > -1:
                return 1 + max_socket_index
        except:
            pass

    # Next try parsing /proc/cpuinfo
    if os.access("/proc/cpuinfo", os.R_OK):
        try:
            lines = open("/proc/cpuinfo", 'r').readlines()
            max_socket_index = -1
            for line in lines:
                if 'physical id' in line:
                    socket_index = int(line.split(':')[1].strip())
                    if socket_index > max_socket_index:
                        max_socket_index = socket_index
            if max_socket_index > -1:
                return 1 + max_socket_index
        except:
            pass

    # Next try dmidecode
    if os.access("/usr/sbin/dmidecode", os.X_OK):
        try:
            lines = os.popen("/usr/sbin/dmidecode -t processor").readlines()
            count = 0
            for line in lines:
                if 'Processor Information' in line:
                    count += 1
            if count > 0:
                return count
        except:
            pass

    return None

# This has got to be one of the ugliest fucntions alive
def read_cpuinfo():
    def get_entry(a, entry):
        e = entry.lower()
        if not a.has_key(e):
            return ""
        return a[e]

    # read cpu list and return number of cpus and list as dictionary
    def get_cpulist_as_dict(cpulist):
        count = 0
        tmpdict = {}
        for cpu in cpulist.split("\n\n"):
            if not len(cpu):
                continue
            count = count + 1
            if count > 1:
                break # no need to parse rest
            for cpu_attr in cpu.split("\n"):
                if not len(cpu_attr):
                    continue
                vals = cpu_attr.split(":")
                if len(vals) != 2:
                    # XXX: make at least some effort to recover this data...
                    continue
                name, value = vals[0].strip(), vals[1].strip()
                tmpdict[name.lower()] = value
        return tmpdict

    if not os.access("/proc/cpuinfo", os.R_OK):
        return {}

    # Okay, the kernel likes to give us the information we need in the
    # standard "C" locale.
    if locale:
        # not really needed if you don't plan on using atof()
        locale.setlocale(locale.LC_NUMERIC, "C")

    cpulist = open("/proc/cpuinfo", "r").read()
    uname = os.uname()[4].lower()
    count = cpu_count()

    # This thing should return a hwdict that has the following
    # members:
    #
    # class, desc (required to identify the hardware device)
    # count, type, model, model_number, model_ver, model_rev
    # bogomips, platform, speed, cache
    
    hwdict = { 'class': "CPU",
               "desc" : "Processor",
               }
    if uname[0] == "i" and uname[-2:] == "86" or (uname == "x86_64"):
        # IA32 compatible enough
        tmpdict = get_cpulist_as_dict(cpulist)

        if uname == "x86_64":
            hwdict['platform'] = 'x86_64'
        else:
            hwdict['platform'] = "i386"
            
        hwdict['count']         = count
        hwdict['type']          = get_entry(tmpdict, 'vendor_id')
        hwdict['model']         = get_entry(tmpdict, 'model name')
        hwdict['model_number']  = get_entry(tmpdict, 'cpu family')
        hwdict['model_ver']     = get_entry(tmpdict, 'model')
        hwdict['model_rev']     = get_entry(tmpdict, 'stepping')
        hwdict['cache']         = get_entry(tmpdict, 'cache size')
        hwdict['bogomips']      = get_entry(tmpdict, 'bogomips')
        hwdict['other']         = get_entry(tmpdict, 'flags')
        mhz_speed               = get_entry(tmpdict, 'cpu mhz')
        if mhz_speed == "":
            # damn, some machines don't report this
            mhz_speed = "-1"
        try:
            hwdict['speed']         = int(round(float(mhz_speed)) - 1)
        except ValueError:
            hwdict['speed'] = -1
    elif uname in["alpha", "alphaev6"]:
        # Treat it as an an Alpha
        tmpdict = get_cpulist_as_dict(cpulist)

        hwdict['platform']      = "alpha"
        hwdict['count']         = get_entry(tmpdict, 'cpus detected')
        hwdict['type']          = get_entry(tmpdict, 'cpu')
        hwdict['model']         = get_entry(tmpdict, 'cpu model')
        hwdict['model_number']  = get_entry(tmpdict, 'cpu variation')
        hwdict['model_version'] = "%s/%s" % (get_entry(tmpdict, 'system type'),
                                             get_entry(tmpdict,'system variation'))
        hwdict['model_rev']     = get_entry(tmpdict, 'cpu revision')
        hwdict['cache']         = "" # pitty the kernel doesn't tell us this.
        hwdict['bogomips']      = get_entry(tmpdict, 'bogomips')
        hwdict['other']         = get_entry(tmpdict, 'platform string')
        hz_speed                = get_entry(tmpdict, 'cycle frequency [Hz]')
        # some funky alphas actually report in the form "462375000 est."
        hz_speed = hz_speed.split()
        try:
            hwdict['speed']         = int(round(float(hz_speed[0]))) / 1000000
        except ValueError:
            hwdict['speed'] = -1
    elif uname in ["ia64"]:
        tmpdict = get_cpulist_as_dict(cpulist)

        hwdict['platform']      = uname
        hwdict['count']         = count
        hwdict['type']          = get_entry(tmpdict, 'vendor')
        hwdict['model']         = get_entry(tmpdict, 'family')
        hwdict['model_ver']     = get_entry(tmpdict, 'archrev')
        hwdict['model_rev']     = get_entry(tmpdict, 'revision')
        hwdict['bogomips']      = get_entry(tmpdict, 'bogomips')
        mhz_speed = tmpdict['cpu mhz']
        try:
            hwdict['speed'] = int(round(float(mhz_speed)) - 1)
        except ValueError:
            hwdict['speed'] = -1
        hwdict['other']         = get_entry(tmpdict, 'features')

    elif uname in ['ppc64']:
        tmpdict = get_cpulist_as_dict(cpulist)

        hwdict['platform'] = uname
        hwdict['count'] = count
        hwdict['model'] = get_entry(tmpdict, "cpu")
        hwdict['model_ver'] = get_entry(tmpdict, 'revision')
        hwdict['bogomips'] = get_entry(tmpdict, 'bogomips')
        hwdict['type'] = get_entry(tmpdict, 'machine')
        # strings are postpended with "mhz"
        mhz_speed = get_entry(tmpdict, 'clock')[:-3]
        try:
            hwdict['speed'] = int(round(float(mhz_speed)) - 1)
        except ValueError:
            hwdict['speed'] = -1
         
    elif uname in ['s390', 's390x']:
        tmpdict = {}
        for cpu in cpulist.split("\n"):
            vals = cpu.split(": ")
            if len(vals) != 2:
                continue
            tmpdict[vals[0].strip()] = vals[1].strip()

        hwdict['platform']      = uname
        hwdict['type']          = get_entry(tmpdict,'vendor_id')
        hwdict['model']         = uname
        hwdict['count']         = count
        hwdict['bogomips']      = get_entry(tmpdict, 'bogomips per cpu')
        hwdict['model_number']  = ""
        hwdict['model_ver']     = ""
        hwdict['model_rev']     = ""
        hwdict['cache']         = ""
        hwdict['other']         = get_entry(tmpdict, 'features')
        hwdict['speed']         = 0

        
    else:
        # XXX: expand me. Be nice to others
        hwdict['platform']      = uname
        hwdict['count']         = count
        hwdict['type']          = uname
        hwdict['model']         = uname
        hwdict['model_number']  = ""
        hwdict['model_ver']     = ""
        hwdict['model_rev']     = ""
        hwdict['cache']         = ""
        hwdict['bogomips']      = ""
        hwdict['other']         = ""
        hwdict['speed']         = 0

    # make sure we get the right number here
    if not hwdict["count"]:
        hwdict["count"] = 1
    else:
        try:
            hwdict["count"] = int(hwdict["count"])
        except:
            hwdict["count"] = 1
        else:
            if hwdict["count"] == 0: # we have at least one
                hwdict["count"] = 1

    # Network communication doesn't really belong in here. Sadly though
    # this is the only single place we can put this check. If it's not
    # here then it would need to be in five or six other places, which
    # is not good from a DRY and quality-assurance perspective.
    s = rhnserver.RhnServer()
    if s.capabilities.hasCapability('cpu_sockets'):
        # If we know it add in the number of sockets
        number_sockets = __get_number_sockets()
        if number_sockets:
            hwdict['socket_count'] = number_sockets
        
    # This whole things hurts a lot.
    return hwdict

def read_memory():
    un = os.uname()
    kernel = un[2]
    if kernel[:3] >= "2.6":
        return read_memory_2_6()
    if kernel[:3] == "2.4":
        return read_memory_2_4()

def read_memory_2_4():
    if not os.access("/proc/meminfo", os.R_OK):
        return {}

    meminfo = open("/proc/meminfo", "r").read()
    lines = meminfo.split("\n")
    curline = lines[1]
    memlist = curline.split()
    memdict = {}
    memdict['class'] = "MEMORY"
    megs = int(long(memlist[1])/(1024*1024))
    if megs < 32:
        megs = megs + (4 - (megs % 4))
    else:
        megs = megs + (16 - (megs % 16))
    memdict['ram'] = str(megs)
    curline = lines[2]
    memlist = curline.split()
    # otherwise, it breaks on > ~4gigs of swap
    megs = int(long(memlist[1])/(1024*1024))
    memdict['swap'] = str(megs)
    return memdict

def read_memory_2_6():
    if not os.access("/proc/meminfo", os.R_OK):
        return {}
    meminfo = open("/proc/meminfo", "r").read()
    lines = meminfo.split("\n")
    meminfo_dict = {}
    for line in lines:
        blobs = line.split(":", 1)
        key = blobs[0]
        if len(blobs) == 1:
            continue
        #print blobs
        value = blobs[1].strip()
        meminfo_dict[key] = value
        
    memdict = {}
    memdict["class"] = "MEMORY"
    
    total_str = meminfo_dict['MemTotal']
    blips = total_str.split(" ")
    total_k = long(blips[0])
    megs = long(total_k/(1024))

    swap_str = meminfo_dict['SwapTotal']
    blips = swap_str.split(' ')
    swap_k = long(blips[0])
    swap_megs = long(swap_k/(1024))

    memdict['ram'] = str(megs)
    memdict['swap'] = str(swap_megs)
    return memdict


def findHostByRoute():
    """ returns [hostname, intf, intf6]

        Where hostname is you FQDN of this machine.
        And intf is numeric IPv4 address. And intf6 is IPv6 address.
    """
    cfg = config.initUp2dateConfig()
    sl = config.getServerlURL()

    st = {'https':443, 'http':80}
    hostname = None
    intf = None
    intf6 = None
    for serverUrl in sl:
        server = serverUrl.split('/')[2]
        servertype = serverUrl.split(':')[0]
        port = st[servertype]

        for family in (AF_INET6, AF_INET):
            try:
                s = socket.socket(family)
            except socket.error:
                continue

            if cfg['enableProxy']:
                server_port = config.getProxySetting()
                (server, port) = server_port.split(':')
                port = int(port)

            try:
                s.settimeout(5)
                s.connect((server, port))
                intf_tmp = s.getsockname()[0]
                if family == AF_INET:
                    intf = intf_tmp
                else:
                    intf6 = intf_tmp
                hostname_tmp = socket.getfqdn(intf_tmp)
                if hostname_tmp != intf_tmp:
                    hostname = hostname_tmp
            except socket.error:
                s.close()
                continue
            s.close()

    # Override hostname with the value from /etc/hostname
    if os.path.isfile("/etc/hostname") and os.access("/etc/hostname", os.R_OK):
        hostnameinfo = open("/etc/hostname", "r").readlines()

        for info in hostnameinfo:
            if not len(info):
                continue
            hostname = info.strip()

    # Override hostname with the one in /etc/sysconfig/network 
    # for bz# 457953
    elif os.path.isfile("/etc/sysconfig/network") and os.access("/etc/sysconfig/network", os.R_OK):
        networkinfo = open("/etc/sysconfig/network", "r").readlines()
	
        for info in networkinfo:
            if not len(info):
                continue
            vals = info.split('=')
            if len(vals) <= 1:
                continue
            strippedstring = vals[0].strip()
            vals[0] = strippedstring
            if vals[0] == "HOSTNAME":
                hostname = ''.join(vals[1:]).strip()
                break
        
    if hostname == None or hostname == 'localhost.localdomain':
        hostname = "unknown"
    return hostname, intf, intf6

def get_slave_hwaddr(master, slave):
    hwaddr = ""
    try:
        bonding = open('/proc/net/bonding/%s' % master, "r")
    except:
        return hwaddr

    slave_found = False
    for line in bonding.readlines():
        if slave_found and line.find("Permanent HW addr: ") != -1:
            hwaddr = line.split()[3]
            break

        if line.find("Slave Interface: ") != -1:
            ifname = line.split()[2]
            if ifname == slave:
                slave_found = True

    bonding.close()
    return hwaddr
    
def read_network():
    netdict = {}
    netdict['class'] = "NETINFO"

    netdict['hostname'], netdict['ipaddr'], netdict['ip6addr'] = findHostByRoute()

    if netdict['hostname'] == "unknown":
        netdict['hostname'] = gethostname()
        if "." not in netdict['hostname']:
            netdict['hostname'] = socket.getfqdn()

    if netdict['ipaddr'] is None:
        try:
            list_of_addrs = getaddrinfo(netdict['hostname'], None)
            ipv4_addrs = filter(lambda x:x[0]==socket.AF_INET, list_of_addrs)
            # take first ipv4 addr
            netdict['ipaddr'] = ipv4_addrs[0][4][0]
        except:
            netdict['ipaddr'] = "127.0.0.1"

    if netdict['ip6addr'] is None:
        try:
            list_of_addrs = getaddrinfo(netdict['hostname'], None)
            ipv6_addrs = filter(lambda x:x[0]==socket.AF_INET6, list_of_addrs)
            # take first ipv6 addr
            netdict['ip6addr'] = ipv6_addrs[0][4][0]
        except:
            netdict['ip6addr'] = "::1"

    if netdict['ipaddr'] is None:
        netdict['ipaddr'] = ''
    if netdict['ip6addr'] is None:
        netdict['ip6addr'] = ''
    return netdict

def read_network_interfaces():
    intDict = {}
    intDict['class'] = "NETINTERFACES"

    if not ethtool_present:
        # ethtool is not available on non-linux platforms (as kfreebsd), skip it
        return intDict

    interfaces = list(set(ethtool.get_devices() + ethtool.get_active_devices()))
    for interface in interfaces:
        try:
            hwaddr = ethtool.get_hwaddr(interface)
        except:
            hwaddr = ""
            
        # slave devices can have their hwaddr changed
        try:
            master = os.readlink('/sys/class/net/%s/master' % interface)
        except:
            master = None

        if master:
            master_interface = os.path.basename(master)
            hwaddr = get_slave_hwaddr(master_interface, interface)

        try:
            module = ethtool.get_module(interface)
        except:
            if interface == 'lo':
                module = "loopback"
            else:
                module = "Unknown"
        try:
            ipaddr = ethtool.get_ipaddr(interface)
        except:
            ipaddr = ""

        try:
            netmask = ethtool.get_netmask(interface)
        except:
            netmask = ""

        try:
            broadcast = ethtool.get_broadcast(interface)
        except:
            broadcast = ""

        ip6_list = []
        dev_info = ethtool.get_interfaces_info(interface)
        for info in dev_info:
            # one interface may have more IPv6 addresses
            for ip6 in info.get_ipv6_addresses():
                scope = ip6.scope
                if scope == 'global':
                    scope = 'universe'
                ip6_list.append({
                    'scope':   scope,
                    'addr':    ip6.address,
                    'netmask': ip6.netmask
                })
        intDict[interface] = {'hwaddr':hwaddr,
                              'ipaddr':ipaddr,
                              'netmask':netmask,
                              'broadcast':broadcast,
                              'module': module,
                              'ipv6': ip6_list}

    return intDict


# Read DMI information via hal.    
def read_dmi():
    dmidict = {}
    dmidict["class"] = "DMI" 

    # Try to obtain DMI info if architecture is i386, x86_64 or ia64
    uname = os.uname()[4].lower()
    if not (uname[0] == "i"  and  uname[-2:] == "86") and not (uname == "x86_64"):
        return dmidict

    # System Information 
    vendor = dmi_vendor()
    if vendor:
        dmidict["vendor"] = vendor
        
    product = get_dmi_data('/dmidecode/SystemInfo/ProductName')
    if product:
        dmidict["product"] = product
        
    version = get_dmi_data('/dmidecode/SystemInfo/Version')
    if version:
        system = product + " " + version
        dmidict["system"] = system

    # BaseBoard Information
    dmidict["board"] = get_dmi_data('/dmidecode/BaseBoardInfo/Manufacturer')

    # Bios Information    
    vendor = get_dmi_data('/dmidecode/BIOSinfo/Vendor')
    if vendor:
        dmidict["bios_vendor"] = vendor
    version = get_dmi_data('/dmidecode/BIOSinfo/Version')
    if version:
        dmidict["bios_version"] = version
    release = get_dmi_data('/dmidecode/BIOSinfo/ReleaseDate')
    if release:
        dmidict["bios_release"] = release

    # Chassis Information
    # The hairy part is figuring out if there is an asset tag/serial number of importance
    chassis_serial = get_dmi_data('/dmidecode/ChassisInfo/SerialNumber')
    chassis_tag = get_dmi_data('/dmidecode/ChassisInfo/AssetTag')
    board_serial = get_dmi_data('/dmidecode/BaseBoardInfo/SerialNumber')
    
    system_serial = get_dmi_data('/dmidecode/SystemInfo/SerialNumber')
    
    dmidict["asset"] = "(%s: %s) (%s: %s) (%s: %s) (%s: %s)" % ("chassis", chassis_serial,
                                                     "chassis", chassis_tag,
                                                     "board", board_serial,
                                                     "system", system_serial)
    
    # Clean up empty entries    
    for k in dmidict.keys()[:]:
        if dmidict[k] is None:
            del dmidict[k]
            # Finished
            
    return dmidict

def get_hal_system_and_smbios():
    try:
        if using_gudev:
            props = get_computer_info()
        else: 
            computer = get_hal_computer()
            props = computer.GetAllProperties()
    except Exception, e:
        log = up2dateLog.initLog()
        msg = "Error reading system and smbios information: %s\n" % (e)
        log.log_debug(msg)
        return {}
    system_and_smbios = {}

    for key in props:
        if key.startswith('system'):
            system_and_smbios[unicode(key)] = unicode(props[key])

    system_and_smbios.update(get_smbios())
    return system_and_smbios

def get_smbios():
    """ Returns dictionary with values we are interested for.
        For historical reason it is in format, which use HAL.
        Currently in dictionary are keys:
        smbios.system.uuid, smbios.bios.vendor, smbios.system.serial,
        smbios.system.manufacturer.
    """
    _initialize_dmi_data()
    if _dmi_not_available:
        return {}
    else:
        return {
            'smbios.system.uuid': dmi_system_uuid(),
            'smbios.bios.vendor': dmi_vendor(),
            'smbios.system.serial': get_dmi_data('/dmidecode/SystemInfo/SerialNumber'),
            'smbios.system.manufacturer': get_dmi_data('/dmidecode/SystemInfo/Manufacturer'),
            'smbios.system.product': get_dmi_data('/dmidecode/SystemInfo/ProductName'),
            'smbios.system.skunumber': get_dmi_data('/dmidecode/SystemInfo/SKUnumber'),
            'smbios.system.family': get_dmi_data('/dmidecode/SystemInfo/Family'),
            'smbios.system.version': get_dmi_data('/dmidecode/SystemInfo/Version'),
        }

# this one reads it all
def Hardware():
    if using_gudev:
        allhw = get_devices()
    else:
        hal_status, dbus_status = check_hal_dbus_status()
        hwdaemon = 1
        if hal_status or dbus_status:
            # if status != 0 haldaemon or messagebus service not running. 
            # set flag and dont try probing hardware and DMI info
            # and warn the user.
            log = up2dateLog.initLog()
            msg = "Warning: haldaemon or messagebus service not running. Cannot probe hardware and DMI information.\n"
            log.log_me(msg)
            hwdaemon = 0
        allhw = []

        if hwdaemon:
            try:
                ret = read_hal()
                if ret: 
                    allhw = ret
            except:
                # bz253596 : Logging Dbus Error messages instead of printing on stdout
                log = up2dateLog.initLog()
                msg = "Error reading hardware information: %s\n" % (sys.exc_type)
                log.log_me(msg)
        
    # all others return individual arrays

    # cpu info
    try:
        ret = read_cpuinfo()
        if ret: allhw.append(ret)
    except:
        print _("Error reading cpu information:"), sys.exc_type
        
    # memory size info
    try:
        ret = read_memory()
        if ret: allhw.append(ret)
    except:
        print _("Error reading system memory information:"), sys.exc_type
        
    cfg = config.initUp2dateConfig()
    if not cfg["skipNetwork"]:
        # minimal networking info
        try:
            ret = read_network()
            if ret: 
                allhw.append(ret)
        except:
            print _("Error reading networking information:"), sys.exc_type
    # dont like catchall exceptions but theres not
    # really anything useful we could do at this point
    # and its been trouble prone enough 

    # minimal DMI info
    try:
        ret = read_dmi()
        if ret:
            allhw.append(ret)
    except:
        # bz253596 : Logging Dbus Error messages instead of printing on stdout
        log = up2dateLog.initLog()
        msg = "Error reading DMI information: %s\n" % (sys.exc_type)
        log.log_me(msg)
        
    try:
        ret = read_installinfo()
        if ret:
            allhw.append(ret)
    except:
        print _("Error reading install method information:"), sys.exc_type

    if not cfg["skipNetwork"]:
        try:
            ret = read_network_interfaces()
            if ret:
                allhw.append(ret)
        except:
            print _("Error reading network interface information:"), sys.exc_type
    
    # all Done.
    return allhw

# XXX: Need more functions here:
#  - filesystems layout (/proc.mounts and /proc/mdstat)
#  - is the kudzu config enough or should we strat chasing lscpi and try to parse that
#    piece of crap output?

#
# Main program
#
if __name__ == '__main__':
    for hw in Hardware():
        for k in hw.keys():
            print "'%s' : '%s'" % (k, hw[k])
        print



#!/usr/bin/python
#
# Spacewalk registration tool
# Adapted from wrapper.py
# Copyright (c) 1999--2012 Red Hat, Inc.  Distributed under GPLv2.
#
# Authors:
#       Adrian Likins <alikins@redhat.com>
#       Preston Brown <pbrown@redhat.com>
#       James Bowes <jbowes@redhat.com> 

import sys

import gettext
t = gettext.translation('rhn-client-tools', fallback=True)
_ = t.ugettext

sys.path.append("/usr/share/rhn/")

from up2date_client import up2dateAuth
from up2date_client import rhncli
from up2date_client import rhnPackageInfo
from up2date_client import rhnHardware

try:
    from virtualization import support
except ImportError:
    support = None    

class ProfileCli(rhncli.RhnCli):

    def main(self):
        if not up2dateAuth.getSystemId():
            needToRegister = \
                _("You need to register this system by running " \
                "`rhn_register` before using this option")
            print needToRegister
            sys.exit(1)

        if not self._testRhnLogin():
            sys.exit(1)

        print _("Updating package profile...")
        rhnPackageInfo.updatePackageProfile()
        
        print _("Updating hardware profile...")
        rhnHardware.updateHardware()
       
        if support is not None:
            print _("Updating virtualization profile...")
            support.refresh(True)


if __name__ == "__main__":
    cli = ProfileCli()
    cli.run()



# Copyright 2011 Red Hat, Inc.
# Copyright 2003 Brent Fox <bfox@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Authors:
#     Miroslav Suchy <msuchy@redhat.com>

import sys
sys.path.append("/usr/share/rhn")
from up2date_client import rhnreg, rhnregGui, rhnserver

import gtk

import gettext
_ = lambda x: gettext.ldgettext("rhn-client-tools", x)

gtk.glade.bindtextdomain("rhn-client-tools")

from firstboot.module import Module
from firstboot.constants import RESULT_SUCCESS, RESULT_FAILURE, RESULT_JUMP

class moduleClass(Module):
    def __init__(self):
        Module.__init__(self)
        self.priority = 108.6
        self.sidebarTitle = _("Select operating system release")
        self.title = _("Select operating system release")
        self.chooseChannel = FirstbootChooseChannelPage()

    def needsNetwork(self):
        return True

    def apply(self, interface, testing=False):
        if testing:
            return RESULT_SUCCESS

        self.chooseChannel.chooseChannelPageApply()
        return RESULT_SUCCESS

    def createScreen(self):
        self.vbox = gtk.VBox(spacing=5)
        self.vbox.pack_start(self.chooseChannel.chooseChannelPageVbox(), True, True)

    def initializeUI(self):
        # populate capability - needef for EUSsupported
        s = rhnserver.RhnServer()
        s.capabilities.validate()

        # this populate zstream channels as side effect
        self.chooseChannel.chooseChannelShouldBeShown()

        self.chooseChannel.chooseChannelPagePrepare()

    def shouldAppear(self):
        return not rhnreg.registered()

class FirstbootChooseChannelPage(rhnregGui.ChooseChannelPage):
    pass



#!/usr/bin/python
#
# Spacewalk / Red Hat Network Classic registration tool
# Adapted from wrapper.py
# Copyright (c) 1999--2012 Red Hat, Inc.  Distributed under GPLv2.
#
# Authors:
#       Adrian Likins <alikins@redhat.com>
#       Preston Brown <pbrown@redhat.com>
#       James Bowes <jbowes@redhat.com> 
#       Daniel Benamy <dbenamy@redhat.com>

import sys
import os

import gettext
t = gettext.translation('rhn-client-tools', fallback=True)
_ = t.ugettext

sys.path.append("/usr/share/rhn/")

from up2date_client import up2dateLog
up2dateLog.initLog().set_app_name('rhn_register')
from up2date_client import up2dateAuth
from up2date_client import rhncli
from up2date_client import tui
from up2date_client import up2dateErrors

class RhnRegister(rhncli.RhnCli):
    """Runs rhn_register. Can run it in gui or tui mode depending on 
    availablility of gui, DISPLAY environment variable, and --nox switch.
    
    """
    def __init__(self):
        super(RhnRegister, self).__init__()
        self.optparser.add_option("--nox", action="store_true", default=False,
            help=_("Do not attempt to use X"))

    def _get_ui(self):
        try:
            if os.access("/usr/share/rhn/up2date_client/gui.py", os.R_OK) and \
               os.environ["DISPLAY"] != "" and \
               not self.options.nox:
                from up2date_client import gui
                self.hasGui = True # Used by base class. Yech.
                return gui
        except:
            pass
        
        return tui

    def main(self):
        """RhnCli (the base class) just sets stuff up and then calls this to run
        the rest of the program.
        
        """
        ui = self._get_ui()
        ui.main()

        # Check to see if the registration worked.
        try:
            if not up2dateAuth.getLoginInfo():
                if not self._testRhnLogin():
                    sys.exit(1)
        except up2dateErrors.RhnServerException:
            sys.exit(1)
        
        # Assuming registration worked, remember to save info (proxy setup,etc)
        # from rhncli
        self.saveConfig()
        sys.exit(0)


if __name__ == "__main__":
    app = RhnRegister()
    app.run()



#!/usr/bin/python

import os

import settestpath

import unittest

from up2date_client import clientCaps
from up2date_client import up2dateAuth

test_clientCaps_d = "etc-sysconfig-rhn/clientCaps.d"

class TestClientCaps(unittest.TestCase):
    def setUp(self):
	self.__setupData()

    def __setupData(self):
        self.caps1 = {"packages.runTransaction":{'version':1, 'value':1},
                      "blippyfoo":{'version':5, 'value':0},
                      "caneatCheese":{'version':1, 'value': 1}
                      }

        self.headerFormat1 = [('X-RHN-Client-Capability', 'caneatCheese(1)=1'),
                              ('X-RHN-Client-Capability', 'packages.runTransaction(1)=1'),
                              ('X-RHN-Client-Capability', 'blippyfoo(5)=0')]

        self.dataKeysSorted1 = ['blippyfoo',
                                'caneatCheese',
                                'packages.runTransaction']
        self.dataValuesSorted1 = [{'version': 5, 'value': 0},
                                  {'version': 1, 'value': 1},
                                  {'version': 1, 'value': 1}]
    def testEmptyInit(self):
        "Verify that the class can be created with no arguments"
        cc = clientCaps.ClientCapabilities()

    def testPopulate(self):
        "Verify the object gets created with an approriate populated data"
        cc = clientCaps.ClientCapabilities()
        len = cc.keys()
        self.assertTrue(len >= 1)

    def testHeaderFormat(self):
        "Verify that headerFormat runs without errors"
        cc = clientCaps.ClientCapabilities()
        res = cc.headerFormat()
        self.assertEquals(type([]), type(res))

    def testHeaderFormatVerify(self):
        "Verify that headerFormat returns proper results"
        cc = clientCaps.ClientCapabilities()
        cc.populate(self.caps1)
        res = cc.headerFormat()
        self.assertEquals(type([]), type(res))
        self.assertTrue(len(res) >= 1)

        for header in res:
            headerName, value = header
            self.assertEqual("X-RHN-Client-Capability", headerName)

        self.assertEqual(res, self.headerFormat1)

    def testDataFormatVerify(self):
        "Verify that populate() creates the internal dict's properly"
        cc = clientCaps.ClientCapabilities()
        cc.populate(self.caps1)

        keys = cc.keys()
        keys.sort()
        self.assertEqual(self.dataKeysSorted1, keys)

        values = cc.values()
        values.sort()
        self.assertEqual(self.dataValuesSorted1, values)


    def testLoadClientCaps(self):
        "Verify that loadClientCaps works"
        blip = clientCaps.loadLocalCaps(test_clientCaps_d)

    def testLoadClientCapsSkipDirs(self):
        "Verify that client caps loads with dirs in /etc/sysconfig/rhn/clientCaps.d,"
        # bugzilla #114322
        dirname= test_clientCaps_d + "/TESTDIR"
        if not os.access(dirname, os.R_OK):
            os.makedirs(dirname)
        try:
            clientCaps.loadLocalCaps(test_clientCaps_d)
            os.rmdir(dirname) 
        except:
            os.rmdir(dirname) 
            self.fail()


class TestLoginWithCaps(unittest.TestCase):
    def testLogin(self):
        "Attempt a login that utilizies capabilties"
        # this doesnt neccesarily seem to relate to caps
        # but it's here as a convient way to test login's
        # at the moment...

        # in the future we could also try override different
        # capbilities, etc...
        res = up2dateAuth.login()


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestClientCaps))
    suite.addTest(unittest.makeSuite(TestLoginWithCaps))
    return suite
       
if __name__ == "__main__":
    unittest.main(defaultTest="suite")



#!/usr/bin/python

import settestpath
import unittest

from up2date_client import rhnPackageInfo



class TestPprint_pkglist(unittest.TestCase):
    def setUp(self):
        self.pkgList1 = [["foo", "1.0", "1", "", "i386", "234234234", "some-channel"],
                         ["bar", "2.0", "2", "9", "i686", "34234234234234", "some-channel"]]

        self.pkgList2 = []
        self.pkgList3 = map(lambda a: a+["othercruft", "morecruft"], self.pkgList1)

        # send it a tuple
        self.pkgList4 = ("foo", "1.0", "1", "", "i386", "234234234", "some-channel")
        self.pkgList5 = list(self.pkgList4)

    def testPprint_pkglist(self):
        """Verify that pprint_pkglist properly formats a package list"""
        res = rhnPackageInfo.pprint_pkglist(self.pkgList1)
        assert res == ['foo-1.0-1', 'bar-2.0-2']

    def testEmptyList(self):
        """Verify that pprint_pkglist properly handles a empty list"""
        res = rhnPackageInfo.pprint_pkglist(self.pkgList2)
        assert res == []

    def testTuple(self):
        """Verify that pprint_pkglist properly handles a single tuple"""
        res = rhnPackageInfo.pprint_pkglist(self.pkgList4)
        assert res == "foo-1.0-1"

    def testSingleList(self):
        """Verify that pprint_pkglist proper handles a single list (IndexError)"""
        try:
            res = rhnPackageInfo.pprint_pkglist(self.pkgList5)
            print res
        except IndexError:
            pass
        else:
            self.fail("expected a IndexError")

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestPprint_pkglist))
    return suite

if __name__ == "__main__":
    unittest.main(defaultTest="suite")



#!/usr/bin/python
#
# Registration client for the Spacewalk for useage with kickstart 
# Copyright (c) 1999--2012 Red Hat, Inc.  Distributed under GPLv2.
#
# Authors:
#       Adrian Likins <alikins@redhat.com>
#       James Bowes <jbowes@redhat.com>
#
#  see the output of "--help" for the valid options. 
#
#  The contact info is in the form or a "key: value" one per line.
#  valid keys are:
#       reg_num, title, first_name, last_name, company,
#       position, address1, address2, city, state, zip,
#       country, phone, fax, contact_email, contact_mail,
#       contact_phone, contact_fax, contact_special,
#       contact_newsletter
#
#
#

import sys
import os
from rhn.connections import idn_pune_to_unicode

import gettext
t = gettext.translation('rhn-client-tools', fallback=True)
_ = t.ugettext

sys.path.append("/usr/share/rhn/")

from up2date_client import rhnreg
from up2date_client import hardware
from up2date_client import pkgUtils
from up2date_client import up2dateErrors
from up2date_client import rhncli


class RegisterKsCli(rhncli.RhnCli):

    def __init__(self):
        super(RegisterKsCli, self).__init__()

        self.optparser.add_option("--profilename", action="store",
            help=_("Specify a profilename")),
        self.optparser.add_option("--username", action="store",
            help=_("Specify a username")),
        self.optparser.add_option("--password", action="store",
            help=_("Specify a password")),
        self.optparser.add_option("--systemorgid", action="store",
            help=_("Specify an organizational id for this system")),
        self.optparser.add_option("--serverUrl", action="store",
            help=_("Specify a url to use as a server")),
        self.optparser.add_option("--sslCACert", action="store",
            help=_("Specify a file to use as the ssl CA cert")),
        self.optparser.add_option("--activationkey", action="store",
            help=_("Specify an activation key")),
        self.optparser.add_option("--use-eus-channel", action="store_true",
            help=_("Subscribe this system to the EUS channel tied to the system's redhat-release")),
        self.optparser.add_option("--contactinfo", action="store_true",
            default=False, help=_("[Deprecated] Read contact info from stdin")),
        self.optparser.add_option("--nohardware", action="store_true",
            default=False, help=_("Do not probe or upload any hardware info")),
        self.optparser.add_option("--nopackages", action="store_true",
            default=False, help=_("Do not profile or upload any package info")),
        self.optparser.add_option("--novirtinfo", action="store_true",
            default=False, help=_("Do not upload any virtualization info")),
        self.optparser.add_option("--norhnsd", action="store_true",
            default=False, help=_("Do not start rhnsd after completion")),
        self.optparser.add_option("--force", action="store_true", default=False,
            help=_("Register the system even if it is already registered")),

    def main(self):
        if self.options.serverUrl:
            rhnreg.cfg.set("serverURL", self.options.serverUrl)
        if self.options.sslCACert:
            rhnreg.cfg.set("sslCACert", self.options.sslCACert)

        if not (self.options.activationkey or 
                (self.options.username and self.options.password)):
            print _("A username and password are required "\
                    "to register a system.")
            sys.exit(-1)

        if rhnreg.registered() and not self.options.force:
            print _("This system is already registered. Use --force to override")
            sys.exit(-1)

        rhnreg.getCaps()
        
        if not self.options.nopackages:
            getArch = 0
            if rhnreg.cfg['supportsExtendedPackageProfile']:
                getArch = 1
            packageList = pkgUtils.getInstalledPackageList(getArch=getArch)
        else:
            packageList = []

        
        hardwareList = hardware.Hardware()
        
        if self.options.profilename:
            profilename = self.options.profilename
        else:
            profilename = RegisterKsCli.__generateProfileName(hardwareList)

        other = {}
        if self.options.systemorgid:
            other['org_id'] = self.options.systemorgid

        # Try to get the virt uuid and put it in "other".
        (virt_uuid, virt_type) = rhnreg.get_virt_info()
        if not virt_uuid is None:
            other['virt_uuid'] = virt_uuid
            other['virt_type'] = virt_type
            
        # If specified, send up the EUS channel label for subscription.
        if self.options.use_eus_channel:
            if self.options.activationkey:
                print _("Usage of --use-eus-channel option with --activationkey is not supported. Please use username and password instead.")
  	        sys.exit(-1)
            if not rhnreg.server_supports_eus():
                print _("The server you are registering against does not support EUS.")
                sys.exit(-1)
            
            channels = rhnreg.getAvailableChannels(self.options.username,
                                                   self.options.password)
            other['channel'] = channels['default_channel']

        try:
            if self.options.activationkey:
                systemId = rhnreg.registerSystem(token = self.options.activationkey,
                                                 profileName = profilename,
                                                 other = other)
            else:
                systemId = rhnreg.registerSystem(self.options.username,
                    self.options.password, profilename, other = other)
        except (up2dateErrors.AuthenticationTicketError,
                up2dateErrors.RhnUuidUniquenessError,
                up2dateErrors.CommunicationError,
                up2dateErrors.AuthenticationOrAccountCreationError), e:
            print "%s" % e.errmsg
            sys.exit(1)
 
        # collect hardware info, inluding hostname
        if not self.options.nohardware:
            rhnreg.sendHardware(systemId, hardwareList)

        if not self.options.nopackages:
            rhnreg.sendPackages(systemId, packageList)

        if self.options.contactinfo:
            print _("Warning: --contactinfo option has been deprecated. Please login to the server web user Interface and update your contactinfo. ")

        # write out the new id
        if isinstance(systemId, unicode):
            rhnreg.writeSystemId(unicode.encode(systemId, 'utf-8'))
        else:
            rhnreg.writeSystemId(systemId)

        # assume successful communication with server
        # remember to save the config options
        rhnreg.cfg.save()

        # Send virtualization information to the server.  We must do this
        # *after* writing out the system id.
        if not self.options.novirtinfo:
            rhnreg.sendVirtInfo(systemId)

        # do this after writing out system id, bug #147513
        if not self.options.norhnsd:
            rhnreg.startRhnsd()

        try:
            present, conf_changed = rhnreg.pluginEnable()
            if not present:
                sys.stderr.write(rhncli.utf8_encode(_("Warning: yum-rhn-plugin is not present, could not enable it.")))
        except IOError, e:
            sys.stderr.write(rhncli.utf8_encode(_("Warning: Could not open /etc/yum/pluginconf.d/rhnplugin.conf\nyum-rhn-plugin is not enabled.\n") + e.errmsg))
        RegisterKsCli.__runRhnCheck(self.options.verbose)

    @staticmethod
    def __generateProfileName(hardwareList):
        hostname = None
        ipaddr = ip6addr = None
        profileName = None
        for hw in hardwareList:
            if hw['class'] == 'NETINFO':
                hostname = hw.get('hostname')
                ipaddr = hw.get('ipaddr')
                ip6addr = hw.get('ipaddr6')
                
        if hostname:
            profileName = idn_pune_to_unicode(hostname)
        elif ipaddr:
            profileName = ipaddr
        elif ip6addr:
            profileName = ip6addr
                
        if not profileName:
            print _("A profilename was not specified, "\
                    "and hostname and IP address could not be determined "\
                    "to use as a profilename, please specify one.")
            sys.exit(-1)

        return profileName

    @staticmethod
    def __runRhnCheck(verbose):
        if verbose:
            os.system("/usr/sbin/rhn_check %s" % '-' + ('v' * verbose))
        else:
            os.system("/usr/sbin/rhn_check")

if __name__ == "__main__":
    cli = RegisterKsCli()
    cli.run()



# -*- coding: utf-8 -*-
#
# String constants for the RHN Register TUI/GUI.
# Copyright (c) 2000--2012 Red Hat, Inc.
#
# Author:
#       James Slagle <jslagle@redhat.com>


import gettext
t = gettext.translation('rhn-client-tools', fallback=True)
_ = t.ugettext

COPYRIGHT_TEXT        = _(u"Copyright © 2006--2010 Red Hat, Inc. All rights reserved.")

# Satellite URL Window
SATELLITE_URL_WINDOW  = _("Enter your Red Hat Satellite URL.")
SATELLITE_URL_TEXT    = _("Please enter the location of your Red Hat Satellite "
                          "server and of its SSL "
                          "certificate. The SSL certificate is only required "
                          "if you will be connecting over https (recommended).")
SATELLITE_URL_PROMPT  = _("Satellite URL:")
SATELLITE_URL_PROMPT2 = _("SSL certificate:")
SATELLITE_REQUIRED    = _("You must enter a valid Satellite URL.")
SSL_REQUIRED          = _("If you are using https you must enter the location "
                          "of a valid SSL certificate.")

# Connect Window
CONNECT_WINDOW        = _("Attempting to contact the Spacewalk server.")
CONNECT_WINDOW_TEXT   = _("We are attempting to contact the Red Hat "
                          "Network server at %s.")
CONNECT_WINDOW_TEXT2  = _("A proxy was specified at %s.")

# Start Window
START_REGISTER_WINDOW = _("System Registration")
START_REGISTER_TEXT   = _("This assistant will guide you through "
                          "connecting your system to "
                          "Red Hat Satellite to receive software "
                          "updates, including "
                          "security updates, to keep your system supported "
                          "and compliant.  "
                          "You will need the following at this time:\n\n"
                          " * A network connection\n"
                          " * Your Red Hat Login & password\n"
                          " * The location of a Red Hat Satellite "
                          "or Proxy\n\n")

# Why Register Window
WHY_REGISTER          = _("Why Should I Connect to RHN? ...")
WHY_REGISTER_WINDOW   = _("Why Register")
WHY_REGISTER_TEXT     = _("Connecting your system to Red Hat Satellite allows you to take full "
                          "advantage of the benefits of a paid subscription, including:")
WHY_REGISTER_SEC      = _("Security & Updates:")
WHY_REGISTER_DLD      = _("Downloads & Upgrades:")
WHY_REGISTER_SUPP     = _("Support:")
WHY_REGISTER_COMP     = _("Compliance:")
WHY_REGISTER_SEC_TXT  = _("Receive the latest software updates, including security updates, keeping this "
                          "Red Hat Enterprise Linux system updated and secure.")
WHY_REGISTER_DLD_TXT  = _("Download installation images for Red Hat Enterprise Linux releases, "
                          "including new releases.")
WHY_REGISTER_SUPP_TXT = _("Access to the technical support experts at Red Hat or Red Hat's partners for help "
                          "with any issues you might encounter with this system.")
WHY_REGISTER_COMP_TXT = _("Stay in compliance with your subscription agreement "
                          "and manage subscriptions "
                          "for systems connected to your account.")
WHY_REGISTER_TIP      = _("Tip: Red Hat values your privacy: "
                          "http://www.redhat.com/legal/privacy_statement.html")
BACK_REGISTER         = _("Take me back to the registration")

# Confirm Quit Window
CONFIRM_QUIT           = _("Software Update Not Set Up")
CONFIRM_QUIT_SURE       = _("Are you sure you don't want to connect your system to Red Hat Satellite? "
                           "You'll miss out on the benefits of a Red Hat Enterprise Linux subscription:\n")
CONFIRM_QUIT_WILLNOT       = _("You will not be able to take advantage of these subscription privileges without connecting "
                           "your system to Red Hat Satellite.\n")
CONTINUE_REGISTERING   = _("Take me back to the setup process.")
REGISTER_LATER2        = _("I'll register later.")

# Info Window
REGISTER_WINDOW   = _("Red Hat Account")
LOGIN_PROMPT      = _("Please enter your login information for the %s Red "
                    "Hat Network Satellite:\n\n")
HOSTED_LOGIN      = _("Red Hat Login:")
LOGIN             = _("Login:")
PASSWORD          = _("Password:")
LOGIN_TIP         = _("Tip: Forgot your login or password?  Contact your "
                      "Satellite's Organization Administrator.")
USER_REQUIRED     = _("Please enter a desired login.")
PASSWORD_REQUIRED = _("Please enter and verify a password.")

# OS Release Window
SELECT_OSRELEASE             = _("Operating System Release Version")
OS_VERSION                   = _("Operating System version:")
MINOR_RELEASE                = _(" Minor Release: ")
LIMITED_UPDATES              = _("Limited Updates Only")
ALL_UPDATES                  = _("All available updates")
CONFIRM_OS_RELEASE_SELECTION = _("Confirm operating system release selection")
CONFIRM_OS_ALL               = _("Your system will be subscribed to the base"
                                 " software channel to receive all available"
                                 " updates.")

# Hardware Window
HARDWARE_WINDOW = _("Create Profile - Hardware")
HARDWARE_WINDOW_DESC1 = _("A Profile Name is a descriptive name that"
                          " you choose to identify this System Profile"
                          " on the Red Hat Satellite web pages. Optionally,"
                          " include a computer serial or identification number.")
HARDWARE_WINDOW_DESC2 = _("Additional hardware information including PCI"
                          " devices, disk sizes and mount points will be"
                          " included in the profile.")
HARDWARE_WINDOW_CHECKBOX = _("Include the following information about hardware"
                             " and network:")

# Packages Window
PACKAGES_WINDOW         = _("Create Profile - Packages")
PACKAGES_WINDOW_DESC1   = _("RPM information is important to determine what"
                          " updated software packages are relevant to this"
                          " system.")
PACKAGES_WINDOW_DESC2   = _("Include RPM packages installed on this system"
                          " in my System Profile")
PACKAGES_WINDOW_UNCHECK = _("You may deselect individual packages by"
                            " unchecking them below.")
PACKAGES_WINDOW_PKGLIST = _("Building Package List")

# Product Window
EMAIL                  = _("*Email Address:")

SYSTEM_ALREADY_SETUP = _("System Already Registered")
SYSTEM_ALREADY_REGISTERED = _("It appears this system has already been set up for software updates:")
SYSTEM_ALREADY_REGISTERED_CONT = _("Are you sure you would like to continue?")

RHSM_SYSTEM_ALREADY_REGISTERED = _("This system has already been registered with RHN Classic/Red Hat Satellite"
				" using Red Hat certficate-based technology.\n\n"
				"The tool you are using is attempting to re-register "
				"using Red Hat Satellite or Red Hat Proxy technology. Red Hat recommends "
				"(except in a few cases) that customers only register with Red Hat Satellie once.\n\n"
				"To learn more about RHN Classic/Red Hat Satellite registration and technologies please consult this"
				" Knowledge Base Article: https://access.redhat.com/kb/docs/DOC-45563")

# Send Window
SEND_WINDOW      = _("Send Profile Information to Red Hat Satellite")
SEND_WINDOW_DESC = _("We are finished collecting information for the System Profile.\n\n"
                     "Press \"Next\" to send this System Profile to Red Hat Satellite.  "
                     "Click \"Cancel\" and no information will be sent.  "
                     "You can run the registration program later by "
                     "typing `rhn_register` at the command line.")

# Sending Window
SENDING_WINDOW = _("Sending Profile to Red Hat Satellite")

# Finish Window
FINISH_WINDOW           = _("Updates Configured")
FINISH_WINDOW_TEXT_TUI  = _("You may now run 'yum update' from this system's "
                            "command line to get the latest "
                            "software updates from Red Hat Satellite. You will need to run this "
                            "periodically to "
                            "get the latest updates. Alternatively, you may configure this "
                            "system for automatic software updates (also known as 'auto errata update') "
                            "via the Red Hat Satellite web interface.  (Instructions for this are in chapter 6 "
                            "of the RHN Reference Guide, available from the 'Help' button in the main Red "
                            "Hat Network Satellite web interface.)")

# Review Window
REVIEW_WINDOW           = _("Review Subscription")
REVIEW_WINDOW_PROMPT    = _("Please review the subscription details below:")
SUB_NUM                 = _("The installation number %s was activated during "
                            "this system's initial connection to Red Hat Satellite.")
SUB_NUM_RESULT          = _("Subscriptions have been activated for the following "
                            "Red Hat products/services:")
CHANNELS_TITLE          = _("Software Channel Subscriptions:")
OK_CHANNELS             = _("This system will receive updates from the "
                            "following software channels:")
CHANNELS_SAT_WARNING    = _("Warning: Only installed product listed above will receive "
                            "updates and support. If you would like "
                            "to receive updates for additional products, please "
                            "login to your satellite web interface "
                            "and subscribe this system to the appropriate "
                            "software channels. See Kbase article"
                            "for more details. "
                            "(http://kbase.redhat.com/faq/docs/DOC-11313)")
YUM_PLUGIN_WARNING		= _("Warning: yum-rhn-plugin is not present, could not enable it.\n"
							"Automatic updates will not work.")
YUM_PLUGIN_CONF_CHANGED = _("Note: yum-rhn-plugin has been enabled.")
YUM_PLUGIN_CONF_ERROR	= _("Warning: An error occurred during enabling yum-rhn-plugin.\n"
							"yum-rhn-plugin is not enabled.\n"
							"Automatic updates will not work.")
FAILED_CHANNELS         = _("You were unable to be subscribed to the following "
                            "software channels because there were insufficient "
                            "subscriptions available in your account:")
NO_BASE_CHANNEL            = _(
"This system was unable to subscribe to any software channels. Your system "
"will not receive any software updates to keep it secure and supported. "
"Contact your Satellite administrator about this problem. Once you make the "
"appropriate active subscriptions available in your account, you may browse "
"to this system's profile in the RHN web interface and subscribe this system "
"to software channels via the software > software channels tab.")
SLOTS_TITLE             = _("Service Level:")
OK_SLOTS                = _("Depending on what Red Hat Satellite modules are associated with a system, you'll "
                            "enjoy different benefits. The following are the "
                            "Red Hat Satellite modules associated with this system:")
SLOTS                   =  SLOTS_TITLE + "\n" + OK_SLOTS + "\n%s"
FAILED_SLOTS            = _("This system was unable to be associated with the "
                            "following RHN module(s) because there were "
                            "insufficient subscriptions available in your account:")
UPDATES                 = _("Update module: per-system updates, email errata "
                            "notifications, errata information")
MANAGEMENT              = _("Management module: automatic updates, systems "
                            "grouping, systems permissions, system package profiling")
PROVISIONING            = _("Provisioning module: bare-metal provisioning, existing state provisioning, "
                            "rollbacks, configuration management")
MONITORING              = _("Monitoring module: pre-defined and custom system "
                            "performance probes, system performance email "
                            "notifications, graphs of system performance")

VIRT = _("Virtualization module: software updates for a limited number of "
        "virtual guests on this system.")

VIRT_PLATFORM = _("Virtualization Platform module: software updates for an "
        "unlimited number virtual guests of this system, access to additional "
        "software channels for guests of this system.")

VIRT_FAILED = _("<b>Warning:</b> Any guest systems you create on this system "
        "and register to RHN will consume Red Hat Enterprise Linux "
        "subscriptions beyond this host system's subscription. You will need "
        "to: (1) make a virtualization or virtualization platform system "
        "entitlement available and (2) apply that system entitlement to this "
        "system in RHN's web interface if you do not want virtual guests of "
        "this system to consume additional subscriptions.")

NO_SYS_ENTITLEMENT         = _("This system was unable to be associated with "
"any RHN service level modules. This system will not receive any software "
"updates to keep it secure and supported. Contace your Satellite administrator "
"about this problem. Once you make the "
"appropriate active subscriptions available in your account, you may browse "
"to this system's profile in the RHN web interface, delete the profile, and "
"re-connect this system to Red Hat Satellite.")
ACTIVATION_KEY          = _("Universal default activation key detected\n"
                            "A universal default activation key was detected in your account. "
                            "This means that a set of properties (software channel subscriptions, "
                            "package installations, system group memberships, etc.) "
                            "for your system's connection to Red Hat Satellite or Red Hat Proxy"
                            "have been determined by the activation key rather than your "
                            "installation number.  "
                            "You may also refer to the RHN Reference Guide, section 6.4.6 for more details "
                            "about activation keys (http://access.redhat.com/knowledge/docs/Red_Hat_Network/)\n"
                            "Universal Default activation key: %s")

# Error Messages.
FATAL_ERROR                = _("Fatal Error")
WARNING                    = _("Warning")
HOSTED_CONNECTION_ERROR    = _("We can't contact the Red Hat Satellite.\n\n"
                               "Double check the location provided - is '%s' correct?\n"
                               "If not, you can correct it and try again.\n\n"
                               "Make sure that the network connection on this system is operational.\n\n"
                               "This system will not be able to successfully receive software updates "
                               "from Red Hat without connecting to a Red Hat Satellite server")

BASECHANNELERROR           = _("Architecture: %s, OS Release: %s, OS "
                               "Version: %s")
SERVER_TOO_OLD             = _("This server doesn't support functionality "
                               "needed by this version of the software update"
                               " setup client. Please try again with a newer "
                               "server.")


SSL_CERT_ERROR_MSG         = _("<b><span size=\"16000\">Incompatible Certificate File</span></b>\n\n"
                               "The certificate you provided, <b>%s</b>, is not compatible with "
                               " the Red Hat Satellite server at <b>%s</b>. You may want to double-check"
                               " that you have provided a valid certificate file."
                               " Are you sure you have provided the correct certificate, and that"
                               " the certificate file has not been corrupted?\n\n"
                               "Please try again with a different certificate file.")

SSL_CERT_EXPIRED           = _("<b><span size=\"12000\">Incompatible Certificate File</span></b>\n\n"
                               " The certificate is expired. Please ensure you have the correct "
                               " certificate and your system time is correct.")

SSL_CERT_FILE_NOT_FOUND_ERRER = _("Please verify the values of sslCACert and serverURL in "
                                  "/etc/sysconfig/rhn/up2date. You can either make the "
                                  "serverURL use http instead of https, or you can "
                                  "download the SSL cert from your Satellite, place it "
                                  "in /usr/share/rhn, and ensure sslCACert points to it.")

ACT_KEY_USAGE_LIMIT_ERROR = _("Problem registering system.\n\n"
                              "A universal default activation key limits the "
                              "number of systems which can connect to "
                              "the RHN organization associated with your "
                              "login. To allow this system to connect, "
                              "please contact your RHN organization "
                              "administrator to increase the number of "
                              "systems allowed to connect or to disable "
                              "this universal default activation key. "
                              "More details can be found in Red Hat "
                              "Knowledgebase Article #7924 at "
                              "http://kbase.redhat.com/faq/FAQ_61_7924.shtm ")

CHANNEL_PAGE_TIP       = _("\n Tip: Minor releases with a '*' are currently"
                           " supported by Red Hat.\n\n")

CHANNEL_PAGE_WARNING = _("Warning:You will not be able to limit this"
                          " system to minor release that is older than"
                          " the recent minor release if you select this"
                          " option.\n")

CONFIRM_OS_WARNING      = _("Your system will be subscribed to %s \n"
                            "base software channel. You will not be\n"
                            "able to move this system to an earlier release\n"
                            "(you will be able to move to a newer release).\n"
                            "Are you sure you would like to continue?")



# Navigation
OK        = _("OK")
ERROR     = _("Error")
NEXT      = _("Next")
BACK      = _("Back")
CANCEL    = _("Cancel")
NO_CANCEL = _("No, Cancel")
YES_CONT  = _("Yes, Continue")
DESELECT  = _("Press <space> to deselect the option.")





#!/usr/bin/python



import sys

import settestpath

from up2date_client import config
from up2date_client import rpcServer

import unittest
from rhn import rpclib

test_up2date = "etc-sysconfig-rhn/up2date"

def write(blip):
    sys.stdout.write("\n%s\n" % blip)


class TestGetServer(unittest.TestCase):
    def setUp(self):
        self.cfg = config.initUp2dateConfig(test_up2date)
        self.origssl = self.cfg['sslCACert']

    def tearDown(self):
        # reset this
        self.cfg['enableProxy'] = 0
        self.cfg['enableAuthProxy'] = 0
        self.cfg['serverURL'] = "https://xmlrpc.rhn.redhat.com/XMLRPC"
        self.cfg['sslCACert'] = self.origssl


    def testGetServerDefault(self):
        "Verify that rpcServer.getServer works with default config and arguments"
        try:
            s = rpcServer.getServer()
        except:
            self.fail("Got an expection calling rpcServer.getServer")

    def __callback(self):
        write("callback called")
        
    def testGetServerDefaultWithRefreshCallback(self):
        "Verify that getServer works with default config and a refreshCallback"
        try:
            s = rpcServer.getServer(refreshCallback=self.__callback)
        except:
            self.fail("Got an expection calling rpcServer.getServer")
        
    def testGetServerRedirectServer(self):
        "Verify that getServer works when talking to a server that redirects"
        self.cfg['serverURL'] = "https://rhn.redhat.com/XMLRPC-REDIRECT"
        try:
            s = rpcServer.getServer()
        except:
            self.fail("Got an expection calling rpcServer.getServer")

    def testGetServerProxy(self):
        "Verify that getServer Works when specifying a proxy"
        sys.path.append("/etc/sysconfig/rhn/")
        proxyUrl = testConfig.anonProxyUrl
        self.cfg['httpProxy'] = proxyUrl
        try:
            s = rpcServer.getServer()
        except:
            self.fail("Got an expection calling rpcServer.getServer")

    def testGetServerAuthProxy(self):
        "Verify that getSerer works when specifying an an auth proxy"
        sys.path.append("/etc/sysconfig/rhn/")
        proxyUrl = testConfig.authProxyUrl
        self.cfg['httpProxy'] = testConfig.authProxyUrl
        self.cfg['proxyUser'] = testConfig.authProxyUser
        self.cfg['proxyPassword'] = testConfig.authProxyPassword
        self.cfg['enableAuthProxy'] = 1
        try:
            s = rpcServer.getServer()
        except:
            self.fail("Got an expection calling rpcServer.getServer")
    
    
    def testGetServerAuthProxyNoPassword(self):
        "Verify that getSerer works when specifying an an auth proxy and no user password"
        sys.path.append("/etc/sysconfig/rhn/")
        proxyUrl = testConfig.authProxyUrl
        self.cfg['httpProxy'] = testConfig.authProxyUrl
        self.cfg['proxyUser'] = testConfig.authProxyUser
        self.cfg['enableAuthProxy'] = 1
        try:
            s = rpcServer.getServer()
        except:
            self.fail("Got an expection calling rpcServer.getServer")

    def testGetServerAuthProxyNoUsername(self):
        "Verify that getSerer works when specifying an an auth proxy and no username"
        sys.path.append("/etc/sysconfig/rhn/")
        proxyUrl = testConfig.authProxyUrl
        self.cfg['httpProxy'] = testConfig.authProxyUrl
        self.cfg['enableAuthProxy'] = 1
        try:
            s = rpcServer.getServer()
        except:
            self.fail("Got an expection calling rpcServer.getServer")

    def testGetServerAuthProxyNoEnableAuthProxy(self):
        "getServer with httpProxy set, but no enableAuthProxy"
        sys.path.append("/etc/sysconfig/rhn/")
        proxyUrl = testConfig.authProxyUrl
        self.cfg['httpProxy'] = testConfig.authProxyUrl
        try:
            s = rpcServer.getServer()
        except:
            self.fail("Got an expection calling rpcServer.getServer")

    def testGetServerNoCaCert(self):
        "getServer with a missing CA cert"
        self.cfg['sslCACert'] =  ["/var/not/likely/to/exist/I/hope/SSL-CA-Cert"]
#        rpcServer.rhns_ca_certs = ["/var/not/likely/to/exist/I/hope/SSL-CA-Cert"]
        try:
            s = rpcServer.getServer()
            write(s)
        except SystemExit:
            pass
        else:
            self.fail("Expected to get a sys.exit(1)")

    def testGetServerMultipleCaCert(self):
        "getServer with a multiple CA certs"
        rpcServer.rhns_ca_certs = ["/usr/share/rhn/RHNS-CA-CERT", "/usr/share/rhn/RHNS-CA-CERT"]
        
        try:
            s = rpcServer.getServer()
        except:
            self.fail("Got an %s expection" % sys.exc_type)



class TestRpcServer500Error(unittest.TestCase):
    def setUp(self):
        
        self.defaultServer = testConfig.brokenServer500error
        import testutils
        testutils.setupConfig("fc1-at-pepsi")

        # cant import config until we put the right stuff in place
        self.cfg = config.initUp2dateConfig(test_up2date)
        self.cfg['serverURL'] = self.defaultServer
        
    def tearDown(self):
        import testutils
        testutils.restoreConfig()

    def testDefaultWelcomeMessage(self):
        "Test welcome_message call with no redirects or proxies"
        s = rpcServer.getServer()
        
        ret = s.registration.welcome_message()

##    def testListPackages(self):
##        "Test that listPackages returns something correctly"
##        import up2dateAuth
##        from repoBackends import up2dateRepo
##        ss = up2dateRepo.ServerSettings()

## #       write("%s" % (ss.settings(),))
##        li = up2dateAuth.getLoginInfo()
##        s = up2dateRepo.getGETServer(li, ss)
##        try:
##            ret = rpcServer.doCall(s.getObsoletes, "rhel-i386-as-3", "123123123")
##        except up2dateErrors.CommunicationError, e:
##            pass
##        else:
##            self.fail("Excpectd to get a CommunicationsError here but did not")
###        write(ret)

class TestRpcServerWelcomeMessage(unittest.TestCase):
    def __init__(self, methodname):
        self.neServerSSL = "https://www.hokeypokeyland.coma/FOO"
        self.neServer = "http://www.hokeypokeyland.coma/FOO"
        self.FourOhFourServerSSL = "https://xmlrpc.rhn.redhat.com/XMLSSDFSD"
        self.FourOhFourServer = "http://xmlrpc.rhn.redhat.com/XMLSSDFSD"
        unittest.TestCase.__init__(self, methodname)
        
    def setUp(self):
        self.cfg = config.initUp2dateConfig(test_up2date)
        self.defaultServer = self.cfg['serverURL']
#        self.defaultServer = "http://SECRET_URL/XMLRPC"
#        write("defaultServer: %s" % self.cfg['serverURL'])


    def tearDown(self):
        self.cfg['serverURL'] = self.defaultServer
        self.cfg['enableProxy'] = 0
        self.cfg['enableAuthProxy'] = 0
        
    def testDefaultWelcomeMessage(self):
        "Test welcome_message call with no redirects or proxies"
        s = rpcServer.getServer()
        s.registration.welcome_message()

    
    def testDefaultWelcomeMessageFailoverNonExistentServer(self):
        "Test welcome_message call faling over from a non existent server"
        self.cfg['serverURL'] = [ self.neServer,
                                 'https://SECRET_URL/XMLRPC']
        s = rpcServer.getServer()
        s.registration.welcome_message()

    def testDefaultWelcomeMessageFailover404Server(self):
        "Test welcome_message call faling over from a server that 404's"
        self.cfg['serverURL'] = [ self.FourOhFourServer,
                                  'https://SECRET_URL/XMLRPC']
        s = rpcServer.getServer()
        s.registration.welcome_message()
        
    def testDefaultWelcomeMessageFailoverNonExistentServerRedirect(self):
        "Test welcome_message call faling over from a non existent server to a redirect"
        self.cfg['serverURL'] = [ self.neServer,
                                 'https://SECRET_URL/XMLRPC-REDIRECT']
        s = rpcServer.getServer()
        s.registration.welcome_message()

    def testDefaultWelcomeMessageFailover404ServerRedirect(self):
        "Test welcome_message call faling over from a non existent server to a redirect"
        self.cfg['serverURL'] = [ self.FourOhFourServer,
                                 'https://SECRET_URL/XMLRPC-REDIRECT']
        s = rpcServer.getServer()
        s.registration.welcome_message()
    
    def testDefaultWelcomeMessageHttpsToHttps(self):
        "Test redirecting https to https"
        self.cfg['serverURL'] = "https://SECRET_URL/XMLRPC-REDIRECT"
        s = rpcServer.getServer()
        s.registration.welcome_message()

    def testDefaultWelcomeMessageHttpToHttps(self):
        "Test redirecting http to https"
        self.cfg['serverURL'] = "http://SECRET_URL/XMLRPC-REDIRECT"
        s = rpcServer.getServer()
        s.registration.welcome_message()

    def testDefaultWelcomeMessageHttpsToHttp(self):
        "Test redirecting https to http"
        self.cfg['serverURL'] = "https://SECRET_URL/XMLRPC-REDIRECT-NOSSL"
        s = rpcServer.getServer()
        try:
            s.registration.welcome_message()
        except rpclib.InvalidRedirectionError:
            pass
        else:
            self.fail("IOError expected here but didnt get it")

    
    def testDefaultWelcomeMessageHttpsToHttpFailovers(self):
        "Test redirecting https to http after failing over from bad servers"
        self.cfg['serverURL'] = [self.neServer,
                                 self.FourOhFourServer,
                                 "https://SECRET_URL/XMLRPC-REDIRECT-NOSSL"]
        s = rpcServer.getServer()

        try:
            s.registration.welcome_message()
        except rpclib.InvalidRedirectionError:
        
            pass
        else:
            self.fail("InvalidRedirectionError expected here but didnt get it")

    def testDefaultWelcomeMessageHttpsToHttpFailoversSSLandNonSSL(self):
        "Test redirecting https to http after failing over from bad servers"
        self.cfg['serverURL'] = [self.neServerSSL,
                                 self.FourOhFourServer,
                                 "https://SECRET_URL/XMLRPC-REDIRECT-NOSSL"]
        s = rpcServer.getServer()

        try:
            s.registration.welcome_message()
        except rpclib.InvalidRedirectionError:
        
            pass
        else:
            self.fail("InvalidRedirectionError expected here but didnt get it")

    def testDefaultWelcomeMessageHttpsToHttpFailoversNonSSLandSSL(self):
        "Test redirecting https to http after failing over from bad servers"
        self.cfg['serverURL'] = [self.neServer,
                                 self.FourOhFourServerSSL,
                                 "https://SECRET_URL/XMLRPC-REDIRECT-NOSSL"]
        s = rpcServer.getServer()

        try:
            s.registration.welcome_message()
        except rpclib.InvalidRedirectionError:
        
            pass
        else:
            self.fail("InvalidRedirectionError expected here but didnt get it")

            
    def testDefaultWelcomeMessageHttpsToHttpFailoversSSL(self):
        "Test redirecting https to http after failing over from bad ssl servers"
        self.cfg['serverURL'] = [self.neServerSSL,
                                 self.FourOhFourServerSSL,
                                 "https://SECRET_URL/XMLRPC-REDIRECT-NOSSL"]
        s = rpcServer.getServer()

        try:
            s.registration.welcome_message()
        except rpclib.InvalidRedirectionError:
        
            pass
        else:
            self.fail("InvalidRedirectionError expected here but didnt get it")
        
    def testDefaultWelcomeMessageHttpToHttp(self):
        "Test redirecting http to http"
        self.cfg['serverURL'] = "http://SECRET_URL/XMLRPC-REDIRECT-NOSSL"
        s = rpcServer.getServer()
        s.registration.welcome_message()



    def testDefaultWelcomeMessageHttpsToHttpsCheckReturnCode(self):
        "Test redirecting https to https and verify its return code"
        self.cfg['serverURL'] = "https://SECRET_URL/XMLRPC-REDIRECT"
        s = rpcServer.getServer()
        s.registration.welcome_message()
        status  = s.get_response_status()
        self.assertEqual(status, 200)

    def testDefaultWelcomeMessageHttpToHttpsCheckReturnCode(self):
        "Test redirecting http to https and verify its return code"
        self.cfg['serverURL'] = "http://SECRET_URL/XMLRPC-REDIRECT"
        s = rpcServer.getServer()
        s.registration.welcome_message()
        status = s.get_response_status()
        self.assertEqual(status, 200)

    def testDefaultWelcomeMessageHttpToHttpCheckReturnCode(self):
        "Test redirecting http to http and verify its return code"
        self.cfg['serverURL'] = "http://SECRET_URL/XMLRPC-REDIRECT-NOSSL"
        s = rpcServer.getServer()
        s.registration.welcome_message()
        status = s.get_response_status()
        self.assertEqual(status, 200)

    def testDefaultWelcomeMessageHttpsToHttpsRedirectsOff(self):
        "Test redirecting https to https and verify it fails with allow_redirect off"
        self.cfg['serverURL'] = "https://SECRET_URL/XMLRPC-REDIRECT"
        s = rpcServer.getServer()
        s.allow_redirect(0)
        try:
            s.registration.welcome_message()
        except  rpclib.InvalidRedirectionError:
            pass
        else:
            self.fail("InvalidRedirectionError expected here but didnt get it")

    def testDefaultWelcomeMessageHttpToHttpsRedirectsOff(self):
        "Test redirecting http to https and verify it fails with allow_redirect off"
        self.cfg['serverURL'] = "http://SECRET_URL/XMLRPC-REDIRECT"
        s = rpcServer.getServer()
        s.allow_redirect(0)
        try:
            s.registration.welcome_message()
        except  rpclib.InvalidRedirectionError:
            pass
        else:
            self.fail("InvalidRedirectionError expected here but didnt get it")
        
    def testDefaultWelcomeMessageHttpToHttpRedirectsOff(self):
        "Test redirecting http to http and verify if tails with allow_redirect off"
        self.cfg['serverURL'] = "http://SECRET_URL/XMLRPC-REDIRECT-NOSSL"
        s = rpcServer.getServer()
        s.allow_redirect(0)
        try:
            s.registration.welcome_message()
        except rpclib.InvalidRedirectionError:
            pass
        else:
            self.fail("InvalidRedirectionError expected here but didnt get it")

    def testDefaultWelcomeMessageHttpsToHttpRedirectsOff(self):
        "Test redirecting https to http and verify it fails with allow_redirect off"
        self.cfg['serverURL'] = "https://SECRET_URL/XMLRPC-REDIRECT-NOSSL"
        s = rpcServer.getServer()
        s.allow_redirect(0)

        try:
            s.registration.welcome_message()
        except rpclib.InvalidRedirectionError:
            pass
        else:
            self.fail("InvalidRedirectionError expected here but didnt get it")

    def testDefaultWelcomeMessageHttpsToHttps(self):
        "Test redirecting https to https"
        self.cfg['serverURL'] = "https://SECRET_URL/XMLRPC-REDIRECT"
        s = rpcServer.getServer()
        s.registration.welcome_message()

    def testDefaultWelcomeMessageHttpToHttpsCheckRedirect(self):
        "Test redirecting http to https and verify redirect()"
        self.cfg['serverURL'] = "http://SECRET_URL/XMLRPC-REDIRECT"
        s = rpcServer.getServer()
        s.registration.welcome_message()
        ret = s.redirected()
        self.assertEqual(ret, "https://SECRET_URL/XMLRPC")
        

    def testDefaultWelcomeMessageHttpsToHttpCheckRedirect(self):
        "Test redirecting https to http and verify redirect()"
        self.cfg['serverURL'] = "https://SECRET_URL/XMLRPC-REDIRECT-NOSSL"
        s = rpcServer.getServer()
        try:
            s.registration.welcome_message()
        except rpclib.InvalidRedirectionError:
            pass
        else:
            self.fail("InvalidRedirectError expected here but didnt get it")
        ret = s.redirected()
        self.assertEqual(ret, "http://SECRET_URL/XMLRPC")

    def testDefaultWelcomeMessageHttpToHttpCheckRedirect(self):
        "Test redirecting http to http and verify redirect()"
        self.cfg['serverURL'] = "http://SECRET_URL/XMLRPC-REDIRECT-NOSSL"
        s = rpcServer.getServer()
        s.registration.welcome_message()
        ret = s.redirected()
        self.assertEqual(ret, "http://SECRET_URL/XMLRPC")

    def testDefaulWelcomeMessageNoRedirectCheckRedirect(self):
        "Test a non redirecting connection and see if redirected() is set"
        self.cfg['serverURL'] = "http://SECRET_URL/XMLRPC"
        s = rpcServer.getServer()
        s.registration.welcome_message()
        ret = s.redirected()
        self.assertEqual(ret, None)

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestRpcServerWelcomeMessage))
    suite.addTest(unittest.makeSuite(TestRpcServer500Error))
    suite.addTest(unittest.makeSuite(TestGetServer))
    return suite

if __name__ == "__main__":
    unittest.main(defaultTest="suite")



# some high level utility stuff for rpm handling

# Client code for Update Agent
# Copyright (c) 1999--2012 Red Hat, Inc.  Distributed under GPLv2.
#
# Author: Preston Brown <pbrown@redhat.com>
#         Adrian Likins <alikins@redhat.com>
#


#
#  FIXME: Some exceptions in here are currently in up2date.py
#         fix by moving to up2dateErrors.py and importing from there
#
#

import os
import rpm
import transaction

import gettext
t = gettext.translation('rhn-client-tools', fallback=True)
_ = t.ugettext

def installedHeaderByKeyword(**kwargs):    
    """ just cause this is such a potentially useful looking method... """
    _ts = transaction.initReadOnlyTransaction()   
    mi = _ts.dbMatch()   
    for keyword in kwargs.keys():   
        mi.pattern(keyword, rpm.RPMMIRE_GLOB, kwargs[keyword])     
        # we really shouldnt be getting multiples here, but what the heck     
    headerList = []      
    for h in mi:    
        headerList.append(h)   
    
    return headerList

def verifyPackages(packages):
    """ given a list of package labels, run rpm -V on them
        and return a dict keyed off that data
    """
    data = {}
    missing_packages = []                                                                            
    # data structure is keyed off package
    # label, with value being an array of each
    # line of the output from -V


    retlist = []
    for package in packages:
        # we have to have at least name...

        # Note: we cant reliable match on epoch, so just
        # skip it... two packages that only diff by epoch is
        # way broken anyway
        keywords = {'name': package[0],
                    'version': package[1],
                    'release': package[2],
                    # we left our epoch
                    'arch': package[4]
                    }
        for key in (keywords.keys()):
            if (keywords[key] == None) or (keywords[key] == ""):
                del(keywords[key])

        headers = installedHeaderByKeyword(**keywords)
	if len(headers) == 0:            
	    missing_packages.append(package)

        for header in headers:
            epoch = header['epoch']
            if epoch == None:
                epoch = ""
            # gpg-pubkey "packages" can have an arch of None, see bz #162701
            arch = header["arch"]
            if arch == None:
                arch = ""
                
            pkg = (header['name'], header['version'],
                   header['release'], epoch,
                   arch)

            # dont include arch in the label if it's a None arch, #162701
            if header["arch"] == "":
                packageLabel = "%s-%s-%s" % (pkg[0], pkg[1], pkg[2])
            else:
                packageLabel = "%s-%s-%s.%s" % (pkg[0], pkg[1], pkg[2], pkg[4])
                
            verifystring = "/usr/bin/rpmverify -V %s" % packageLabel
                                                                                
            fd = os.popen(verifystring)
            res = fd.readlines()
            fd.close()
                                                                                
            reslist = []
            for line in res:
                reslist.append(line.strip())
            retlist.append([pkg, reslist])

    return retlist, missing_packages

#FIXME: this looks like a good candidate for caching, since it takes a second
# or two to run, and I can call it a couple of times
def getInstalledPackageList(msgCallback = None, progressCallback = None,
                            getArch=None, getInfo = None):
    """ Return list of packages. Package is hash with keys name, epoch,
        version, release and optionaly arch and cookie
    """
    pkg_list = []
    
    if msgCallback != None:
        msgCallback(_("Getting list of packages installed on the system"))
 
    _ts = transaction.initReadOnlyTransaction()   
    count = 0
    total = 0
    
    for h in _ts.dbMatch():
        if h == None:
            break
        count = count + 1
    
    total = count
    
    count = 0
    for h in _ts.dbMatch():
        if h == None:
            break
        package = {
            'name': h['name'],
            'epoch': h['epoch'],
            'version': h['version'],
            'release': h['release'],
            'installtime': h['installtime']
        }
        if package['epoch'] == None:
            package['epoch'] = ""
        else: # convert it to string
            package['epoch'] = "%s" % package['epoch']
        if getArch:
            package['arch'] = h['arch']
            # the arch on gpg-pubkeys is "None"...
            if package['arch']:
                pkg_list.append(package)
        elif getInfo:
            if h['arch']:
                package['arch'] = h['arch']
            if h['cookie']:
                package['cookie'] = h['cookie']
            pkg_list.append(package)
        else:
            pkg_list.append(package)
        
        if progressCallback != None:
            progressCallback(count, total)
        count = count + 1
    
    pkg_list.sort(key=lambda x:(x['name'], x['epoch'], x['version'], x['release']))
    return pkg_list

def setDebugVerbosity():
    """Set rpm's verbosity mode
    """
    try:
        rpm.setVerbosity(rpm.RPMLOG_DEBUG)
    except AttributeError:
        print "extra verbosity not supported in this version of rpm"



# rhn_register.py - GUI front end code for firstboot screen resolution
#
# Copyright 2003 Red Hat, Inc.
# Copyright 2003 Brent Fox <bfox@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import gtk
import gobject
import sys
import os
import functions

import gnome, gnome.ui
from gtk import glade

from rhn_register_firstboot_gui_window import RhnRegisterFirstbootGuiWindow
sys.path.insert(0, "/usr/share/rhn/up2date_client/")
sys.path.insert(1,"/usr/share/rhn")

import rhnreg
import rhnregGui
import up2dateErrors
import messageWindow

import gettext
t = gettext.translation('rhn-client-tools', fallback=True)
_ = t.ugettext
gtk.glade.bindtextdomain("rhn-client-tools")


class RhnCreateProfileWindow(RhnRegisterFirstbootGuiWindow, rhnregGui.CreateProfilePage):
    #You must specify a runPriority for the order in which you wish your module to run
    runPriority = 108.7
    moduleName = _("Create Profile")
    windowTitle =  moduleName
    shortMessage = _("Connect to Red Hat Satellite")
    needsparent = 1
    needsnetwork = 1
    noSidebar = True

    def __init__(self):
        RhnRegisterFirstbootGuiWindow.__init__(self)
        rhnregGui.CreateProfilePage.__init__(self)
        if rhnreg.registered():
            self.skipme = True

    def updatePage(self):
        self.createProfilePagePrepare()
    
    def _getVbox(self):
        return self.createProfilePageVbox()

    def apply(self, *args):

        ret =  self.createProfilePageVerify()
        if ret:
            return None

        ret = self.createProfilePageApply()
        if ret:
            return None
        
        return True

childWindow = RhnCreateProfileWindow




# all the crap that is stored on the rhn side of stuff
# updating/fetching package lists, channels, etc

import up2dateAuth
import up2dateErrors
import config
import rhnserver

import gettext
t = gettext.translation('rhn-client-tools', fallback=True)
_ = t.ugettext

# heh, dont get much more generic than this...
class rhnChannel:
    # shrug, use attributes for thetime being
    def __init__(self, **kwargs):
        self.dict = {}

        for kw in kwargs.keys():
            self.dict[kw] = kwargs[kw]
               
    def __getitem__(self, item):
        return self.dict[item]

    def __setitem__(self, item, value):
        self.dict[item] = value

    def keys(self):
        return self.dict.keys()

    def values(self):
        return self.dict.values()

    def items(self):
        return self.dict.items()

class rhnChannelList:
    def __init__(self):
        # probabaly need to keep these in order for
        #precedence
        self.list = []

    def addChannel(self, channel):
        self.list.append(channel)


    def channels(self):
        return self.list

    def getByLabel(self, channelname):
        for channel in self.list:
            if channel['label'] == channelname:
                return channel
    def getByName(self, channelname):
        return self.getByLabel(channelname)

    def getByType(self, type):
        channels = []
        for channel in self.list:
            if channel['type'] == type:
                channels.append(channel)
        return channels

# for the gui client that needs to show more info
# maybe we should always make this call? If nothing
# else, wrapper should have a way to show extended channel info
def getChannelDetails(timeout=None):

    channels = []
    sourceChannels = getChannels(timeout=timeout)

    for sourceChannel in sourceChannels.channels():
        if sourceChannel['type'] != 'up2date':
            # FIMXE: kluge since we dont have a good name, maybe be able to fix
            sourceChannel['name'] = sourceChannel['label']
            sourceChannel['description'] = "%s channel %s from  %s" % (sourceChannel['type'],
                                                                           sourceChannel['label'],
                                                                           sourceChannel['url'])
        channels.append(sourceChannel)
    return channels

cmdline_pkgs = []

global selected_channels
selected_channels = None
def getChannels(force=None, label_whitelist=None, timeout=None):
    """ return rhnChannelList containing list of channel we are subscribed to """
    cfg = config.initUp2dateConfig()
    global selected_channels
    if not selected_channels and not force:
        selected_channels = rhnChannelList()
        s = rhnserver.RhnServer(timeout=timeout)

        if not up2dateAuth.getSystemId():
            raise up2dateErrors.NoSystemIdError(_("Unable to Locate SystemId"))

        up2dateChannels = s.up2date.listChannels(up2dateAuth.getSystemId())

        for chan in up2dateChannels:
            if label_whitelist and not label_whitelist.has_key(chan['label']):
                continue
                
            channel = rhnChannel(type = 'up2date', url = config.getServerlURL())
            for key in chan.keys():
                if key == "last_modified":
                    channel['version'] = chan['last_modified']
                else:
                    channel[key] = chan[key]
            selected_channels.addChannel(channel)

    if len(selected_channels.list) == 0:
        raise up2dateErrors.NoChannelsError(_("This system may not be updated until it is associated with a channel."))

    return selected_channels
            

def setChannels(tempchannels):
    global selected_channels
    selected_channels = None
    whitelist = dict(map(lambda x: (x,1), tempchannels))
    return getChannels(label_whitelist=whitelist)



def subscribeChannels(channels,username,passwd):
    s = rhnserver.RhnServer()
    return s.up2date.subscribeChannels(up2dateAuth.getSystemId(), channels, username,
        passwd)

def unsubscribeChannels(channels,username,passwd):
    s = rhnserver.RhnServer()
    return s.up2date.unsubscribeChannels(up2dateAuth.getSystemId(), channels,
        username, passwd)



#
# Progress bar for Update Agent
# Copyright (c) 1999--2012 Red Hat, Inc.
#
# Author: Preston Brown <pbrown@redhat.com>

import gtk
gtk.glade.bindtextdomain("rhn-client-tools")

class Progress:
    def __init__(self):
        glade_prefix = "/usr/share/rhn/up2date_client/"

        self.xml = gtk.glade.XML(glade_prefix + "progress.glade", "progressWindow",
                domain="rhn-client-tools")
        self.progressWindow = self.xml.get_widget("progressWindow")
        self.progressWindow.connect("delete-event", self.progressWindow.hide)
        #self.progressWindow.connect("hide", self.progressWindow.hide)
        cursor = gtk.gdk.Cursor(gtk.gdk.WATCH)
        self.progressWindow.window.set_cursor(cursor)
        while gtk.events_pending():
            gtk.main_iteration(False)

        self.lastProgress = 0.0

    def hide(self):
        self.progressWindow.hide()
        while gtk.events_pending():
            gtk.main_iteration(False)

        del self

    def setLabel(self, text):
        label = self.xml.get_widget("progressLabel")
        label.set_text(text)
        while gtk.events_pending():
            gtk.main_iteration(False)

    # the xmlrpc callbacks only use the first three
    # the GET style use all 4, so pass em but dont use them
    def setProgress(self, amount, total, speed = 0, secs = 0):
        if total:
            i = float(amount) / total
        else:
            i = 1

        if i > 1:
            i = 1
        if i > self.lastProgress + .01 or i == 1:
            self.xml.get_widget("progressBar").set_fraction(i)
            if i == 1:
                # reset
                i = 0
#            gtk.gdk_flush()
            while gtk.events_pending():
                gtk.main_iteration(False)
            self.lastProgress = i

    def setStatusLabel(self, text):
        self.xml.get_widget("statusLabel").set_text(text)
        while gtk.events_pending():
            gtk.main_iteration(False)

    def destroy(self):
        while gtk.events_pending():
            gtk.main_iteration(False)

        self.progressWindow.destroy()

    def noop(self, win, event):
        return True




#!/usr/bin/python

# Client code for Update Agent
# Copyright (c) 1999--2012 Red Hat, Inc.  Distributed under GPLv2.
#
# Author: Adrian Likins <alikins@redhat.com
#

import sys
import os
sys.path.append("/usr/share/rhn/")

__rhnexport__ = [
    'reboot']

from up2date_client import up2dateLog
from up2date_client import config

cfg = config.initUp2dateConfig()
log = up2dateLog.initLog()

# action version we understand
ACTION_VERSION = 2 

def reboot(test=None, cache_only=None):
    if cache_only:
        return (0, "no-ops for caching", {})

    if cfg['noReboot']:
        return (38, "Up2date is configured not to allow reboots", {})
    
    pid = os.fork()
    data = {'version': '0'}
    if not pid:
        try:
            if test:
                os.execvp("/sbin/shutdown", ['/sbin/shutdown','-r','-k', '+3'])
            else:
                os.execvp("/sbin/shutdown", ['/sbin/shutdown','-r', '+3'])
        except OSError:
            data['name'] = "reboot.reboot.shutdown_failed"
            return (34, "Could not execute /sbin/shutdown", data)

    log.log_me("Rebooting the system now")
    # no point in waiting around

    return (0, "Reboot sucessfully started", data)


def main():
    print reboot(test=1)

if __name__ == "__main__":
    main()



#   rhn-client-tools - RHN support tools and libraries
#
# Copyright (C) 2006--2012 Red Hat, Inc.
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; version 2 of the License.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
#   02110-1301  USA

import sys

# Adjust path so we can see the src modules running from branch as well
# as test dir:
sys.path.insert(0, './src/')
sys.path.insert(0, '../src/')
sys.path.insert(0, '../../src/')



#
# $Id$

import rhnserver
import config
import os
import up2dateErrors
import up2dateUtils
import string
import up2dateLog
import clientCaps
import pickle
import time

from types import DictType

from rhn import rpclib

loginInfo = None
pcklAuthFileName = "/var/spool/up2date/loginAuth.pkl"

def getSystemId():
    cfg = config.initUp2dateConfig()
    path = cfg["systemIdPath"]
    if not os.access(path, os.R_OK):
        return None
    
    f = open(path, "r")
    ret = f.read()
        
    f.close()
    return ret

# if a user has upgraded to a newer release of Red Hat but still
# has a systemid from their older release, they need to get an updated
# systemid from the RHN servers.  This takes care of that.
def maybeUpdateVersion():
    cfg = config.initUp2dateConfig()
    try:
        idVer = rpclib.xmlrpclib.loads(getSystemId())[0][0]['os_release']
    except:
        # they may not even have a system id yet.
        return 0

    systemVer = up2dateUtils.getVersion()
    
    if idVer != systemVer:
      s = rhnserver.RhnServer()
    
      newSystemId = s.registration.upgrade_version(getSystemId(), systemVer)

      path = cfg["systemIdPath"]
      dir = path[:string.rfind(path, "/")]
      if not os.access(dir, os.W_OK):
          try:
              os.mkdir(dir)
          except:
              return 0
      if not os.access(dir, os.W_OK):
          return 0

      if os.access(path, os.F_OK):
          # already have systemid file there; let's back it up
          savePath = path + ".save"
          try:
              os.rename(path, savePath)
          except:
              return 0

      f = open(path, "w")
      f.write(newSystemId)
      f.close()
      try:
          os.chmod(path, 0600)
      except:
          pass


def writeCachedLogin():
    """
    Pickle loginInfo to a file
    Returns:
    True    -- wrote loginInfo to a pickle file
    False   -- did _not_ write loginInfo to a pickle file
    """
    log = up2dateLog.initLog()
    log.log_debug("writeCachedLogin() invoked")
    if not loginInfo:
        log.log_debug("writeCachedLogin() loginInfo is None, so bailing.")
        return False
    data = {'time': time.time(),
            'loginInfo': loginInfo}
    
    pcklDir = os.path.dirname(pcklAuthFileName)
    if not os.access(pcklDir, os.W_OK):
        try:
            os.mkdir(pcklDir)
            os.chmod(pcklDir, 0700)
        except:
            log.log_me("Unable to write pickled loginInfo to %s" % pcklDir)
            return False
    pcklAuth = open(pcklAuthFileName, 'wb')
    os.chmod(pcklAuthFileName, 0600)
    pickle.dump(data, pcklAuth)
    pcklAuth.close()
    expireTime = data['time'] + float(loginInfo['X-RHN-Auth-Expire-Offset'])
    log.log_debug("Wrote pickled loginInfo at ", data['time'], " with expiration of ", 
            expireTime, " seconds.")
    return True

def readCachedLogin():
    """
    Read pickle info from a file
    Caches authorization info for connecting to the server.
    """
    log = up2dateLog.initLog()
    log.log_debug("readCachedLogin invoked")
    if not os.access(pcklAuthFileName, os.R_OK):
        log.log_debug("Unable to read pickled loginInfo at: %s" % pcklAuthFileName)
        return False
    pcklAuth = open(pcklAuthFileName, 'rb')
    try:
        data = pickle.load(pcklAuth)
    except EOFError:
        log.log_debug("Unexpected EOF. Probably an empty file, \
                       regenerate auth file")
        pcklAuth.close()
        return False
    pcklAuth.close()
    # Check if system_id has changed
    try:
        idVer = rpclib.xmlrpclib.loads(getSystemId())[0][0]['system_id']
        cidVer = "ID-%s" % data['loginInfo']['X-RHN-Server-Id']
        if idVer != cidVer:
            log.log_debug("system id version changed: %s vs %s" % (idVer, cidVer))
	    return False
    except:
	pass
    createdTime = data['time']
    li = data['loginInfo']
    currentTime = time.time()
    expireTime = createdTime + float(li['X-RHN-Auth-Expire-Offset'])
    #Check if expired, offset is stored in "X-RHN-Auth-Expire-Offset" 
    log.log_debug("Checking pickled loginInfo, currentTime=", currentTime,
            ", createTime=", createdTime, ", expire-offset=", 
            float(li['X-RHN-Auth-Expire-Offset']))
    if (currentTime > expireTime):
        log.log_debug("Pickled loginInfo has expired, created = %s, expire = %s." \
                %(createdTime, expireTime))
        return False
    _updateLoginInfo(li)
    log.log_debug("readCachedLogin(): using pickled loginInfo set to expire at ", expireTime)
    return True

def _updateLoginInfo(li):
    """
    Update the global var, "loginInfo"
    """
    global loginInfo
    if type(li) == DictType:
        if type(loginInfo) == DictType:
            # must retain the reference.
            loginInfo.update(li)
        else:
            # this had better be the initial login or we lose the reference.
            loginInfo = li
    else:
        loginInfo = None

# allow to pass in a system id for use in rhnreg
# a bit of a kluge to make caps work correctly
def login(systemId=None, forceUpdate=False, timeout=None):
    log = up2dateLog.initLog()
    log.log_debug("login(forceUpdate=%s) invoked" % (forceUpdate))
    if not forceUpdate and not loginInfo:
        if readCachedLogin():
            return loginInfo

    server = rhnserver.RhnServer(timeout=timeout)

    # send up the capabality info
    headerlist = clientCaps.caps.headerFormat()
    for (headerName, value) in headerlist:
        server.add_header(headerName, value)

    if systemId == None:
        systemId = getSystemId()

    if not systemId:
        return None
        
    maybeUpdateVersion()
    log.log_me("logging into up2date server")

    li = server.up2date.login(systemId)
    
    # figure out if were missing any needed caps
    server.capabilities.validate()
    _updateLoginInfo(li) #update global var, loginInfo
    writeCachedLogin() #pickle global loginInfo

    if loginInfo:
        log.log_me("successfully retrieved authentication token "
                   "from up2date server")

    log.log_debug("logininfo:", loginInfo)
    return loginInfo

def updateLoginInfo(timeout=None):
    log = up2dateLog.initLog()
    log.log_me("updateLoginInfo() login info")
    # NOTE: login() updates the loginInfo object
    login(forceUpdate=True, timeout=timeout)
    if not loginInfo:
        raise up2dateErrors.AuthenticationError("Unable to authenticate")
    return loginInfo


def getLoginInfo(timeout=None):
    global loginInfo
    try:
        loginInfo = loginInfo
    except NameError:
        loginInfo = None
    if loginInfo:
        return loginInfo
    # NOTE: login() updates the loginInfo object
    login(timeout=timeout)
    return loginInfo




# Client code for Update Agent
# Copyright (c) 1999--2012 Red Hat, Inc.  Distributed under GPLv2.
#
# Author: Preston Brown <pbrown@redhat.com>
#         Adrian Likins <alikins@redhat.com>
#
"""utility functions for up2date"""

import os
import string
import up2dateErrors
import config
import gettext
from pkgplatform import getPlatform
t = gettext.translation('rhn-client-tools', fallback=True)
_ = t.ugettext

if getPlatform() == 'deb':
    import lsb_release
    def _getOSVersionAndRelease():
        dist_info = lsb_release.get_distro_information()
        os_name = dist_info['ID']
        os_version = 'n/a'
        if 'CODENAME' in dist_info:
            os_version = dist_info['CODENAME']
        os_release = dist_info['RELEASE']
        return os_name, os_version, os_release

else:
    import transaction
    def _getOSVersionAndRelease():
        ts = transaction.initReadOnlyTransaction()
        for h in ts.dbMatch('Providename', "redhat-release"):
            osVersionRelease = (h['name'], h['version'], h['release'])
            return osVersionRelease
        else:
            for h in ts.dbMatch('Providename', "distribution-release"):
                osVersionRelease = (h['name'], h['version'], h['release'])
                # zypper requires a exclusive lock on the rpmdb. So we need
                # to close it here.
                ts.ts.closeDB()
                return osVersionRelease
            else:
                raise up2dateErrors.RpmError(
                    "Could not determine what version of Red Hat Linux you "\
                    "are running.\nIf you get this error, try running \n\n"\
                    "\t\trpm --rebuilddb\n\n")

def getVersion():
    '''
    Returns the version of redhat-release rpm
    '''
    cfg = config.initUp2dateConfig()
    if cfg["versionOverride"]:
        return str(cfg["versionOverride"])
    os_release, version, release = _getOSVersionAndRelease()
    return version

def getOSRelease():
    '''
    Returns the name of the redhat-release rpm
    '''
    os_release, version, release = _getOSVersionAndRelease()
    return os_release

def getRelease():
    '''
    Returns the release of the redhat-release rpm
    '''
    os_release, version, release = _getOSVersionAndRelease()
    return release

def getArch():
    if os.access("/etc/rpm/platform", os.R_OK):
        fd = open("/etc/rpm/platform", "r")
        platform = string.strip(fd.read())

        #bz 216225
        #handle some replacements..
        replace = {"ia32e-redhat-linux": "x86_64-redhat-linux"}
        if replace.has_key(platform):
            platform = replace[platform]
        return platform
    arch = os.uname()[4]
    if getPlatform() == 'deb':
        # On debian we only support i386
        if arch in ['i486', 'i586', 'i686']:
            arch = 'i386'
        if arch == 'x86_64':
            arch = 'amd64'
        arch += '-debian-linux'
    return arch

def version():
    # substituted to the real version by the Makefile at installation time.
    return "@VERSION@"



# Copyright 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Authors:
#     Daniel Benamy <dbenamy@redhat.com>

import sys
sys.path.append("/usr/share/rhn/up2date_client/")
sys.path.append("/usr/share/rhn")
import rhnreg
import rhnregGui
from rhn_register_firstboot_gui_window import RhnRegisterFirstbootGuiWindow
import config

import gtk
from gtk import glade
import gettext
t = gettext.translation('rhn-client-tools', fallback=True)
_ = t.ugettext
gtk.glade.bindtextdomain("rhn-client-tools")


class RhnFinishWindow(RhnRegisterFirstbootGuiWindow, rhnregGui.FinishPage):
    runPriority=109
    moduleName = _("Finish Updates Setup")
    windowTitle = moduleName
    shortMessage = _("Connect to Red Hat Satellite")
    needsparent = 1
    needsnetwork = 1
    noSidebar = True
    
    def __init__(self):
        RhnRegisterFirstbootGuiWindow.__init__(self)
        rhnregGui.FinishPage.__init__(self)
        if rhnreg.registered():
            self.skipme = True
    
    def _getVbox(self):
        return self.finishPageVbox()
    
    def updatePage(self):
##        if rhnregGui.hasBaseChannelAndUpdates():
##            self.druid.finish.set_label(_("_Finish"))
##            title = _("Finish setting up software updates")
##        else:
##            self.druid.finish.set_label(_("_Exit registration process"))
##            title = _("Registration unsuccessful")
        self.finishPagePrepare()
##        self.mainWin.set_title(title)
##        self.finishPage.set_title(title)
    
    def apply(self, *args):
        """Returns None to stay on the same page. Anything else will cause 
        firstboot to advance but True is generally used. This is different from 
        the gnome druid in rhn_register.
        
        """
        up2DateConfig = config.initUp2dateConfig()
        up2DateConfig.save()
        return True


childWindow = RhnFinishWindow



#!/usr/bin/python

# Copyright (c) 1999--2012 Red Hat, Inc.  Distributed under GPLv2.
#
# Author: Adrian Likins <alikins@redhat.com>
#

import string
import os

# mark this module as acceptable
__rhnexport__ = [
    'configure',
]

def __configRhnsd(interval, cache_only=None):
    rhnsdconfig = "/etc/sysconfig/rhn/rhnsd"
    fd = open(rhnsdconfig, "r")
    lines = fd.readlines()
    count = 0
    index = None
    tmplines = []
    for line in lines:
        tmp = string.strip(line)
        tmplines.append(tmp)
        comps = string.split(tmp, "=", 1)
        if comps[0] == "INTERVAL":
            index = count
        count = count + 1
            
    if index != None:
        tmplines[index] = "INTERVAL=%s" % interval

    fd.close()
    fd = open(rhnsdconfig, "w")
    contents = string.join(tmplines, "\n")
    fd.write(contents)
    fd.close()


def configure(interval=None, restart=None, cache_only=None):
    if cache_only:
        return (0, "no-ops for caching", {})
    msg = ""
    if interval:
        try:
            __configRhnsd(interval)
            msg = "rhnsd interval config updated. "
        except IOError:
            # i'm runing as root, must of been chattr'ed.
            # i'll resist the erge to unchattr this file
            return (37,"Could not modify /etc/sysconfig/rhn/rhnsd", {})

    if restart:
        rc = os.system("/sbin/service rhnsd restart > /dev/null")
        msg = msg + "rhnsd restarted"

    return(0,  msg, {})


if __name__ == "__main__":
    print configure("240")

    print configure("361", 1)

    print configure("127", restart=1)

    print configure(restart=1)

    print configure("192")
